;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                                                              ;;
;; Copyright (C) KolibriOS team 2004-2007. All rights reserved. ;;
;; Distributed under terms of the GNU General Public License    ;;
;;                                                              ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

format MS COFF

DEBUG		equ 1

include 'proc32.inc'
include 'imports.inc'
include 'emu10k1.inc'
include 'emupcm.inc'
include 'io.inc'

API_VERSION	equ 0x01000100

USE_COM_IRQ	equ 0	 ;make irq 3 and irq 4 available for PCI devices
IRQ_REMAP	equ 0
IRQ_LINE	equ 0


;irq 0,1,2,8,12,13 ??????
;                   FEDCBA9876543210
VALID_IRQ	equ 1100111011111000b
ATTCH_IRQ	equ 0000111010100000b

if USE_COM_IRQ
ATTCH_IRQ	equ 0000111010111000b
end if

CPU_FREQ	equ  2600d

BIT0  EQU 0x00000001
BIT1  EQU 0x00000002
BIT5  EQU 0x00000020
BIT10 EQU 0x00000400

VID_Creative	   equ 0x1102

CTRL_EMU10K1	   equ 0x0002  ; EMU10K1
CTRL_Audigy	   equ 0x0004  ; Audigy
CTRL_Audigy2	   equ 0x0008  ; Audigy 2 Value SB0400


CODEC_MASTER_VOL_REG	     equ 0x02
CODEC_AUX_VOL		     equ 0x04 ;
CODEC_PCM_OUT_REG	     equ 0x18 ; PCM output volume
CODEC_EXT_AUDIO_REG	     equ 0x28 ; extended audio
CODEC_EXT_AUDIO_CTRL_REG     equ 0x2a ; extended audio control
CODEC_PCM_FRONT_DACRATE_REG  equ 0x2c ; PCM out sample rate
CODEC_PCM_SURND_DACRATE_REG  equ 0x2e ; surround sound sample rate
CODEC_PCM_LFE_DACRATE_REG    equ 0x30 ; LFE sample rate


SRV_GETVERSION	      equ  0
DEV_PLAY	      equ  1
DEV_STOP	      equ  2
DEV_CALLBACK	      equ  3
DEV_SET_BUFF	      equ  4
DEV_NOTIFY	      equ  5
DEV_SET_MASTERVOL     equ  6
DEV_GET_MASTERVOL     equ  7
DEV_GET_INFO	      equ  8

struc AC_CNTRL		    ;AC controller base class
{ .bus		      dd ?
  .devfn	      dd ?

  .vendor	      dd ?
  .dev_id	      dd ?
  .pci_cmd	      dd ?
  .pci_stat	      dd ?

  .codec_io_base      dd ?
  .codec_mem_base     dd ?

  .ctrl_io_base       dd ?
  .ctrl_mem_base      dd ?
  .cfg_reg	      dd ?
  .int_line	      dd ?

  .vendor_ids	      dd ?    ;vendor id string
  .ctrl_ids	      dd ?    ;hub id string

  .buffer	      dd ?

  .notify_pos	      dd ?
  .notify_task	      dd ?

  .lvi_reg	      dd ?
  .ctrl_setup	      dd ?
  .user_callback      dd ?
  .codec_read16       dd ?
  .codec_write16      dd ?

  .ctrl_read8	      dd ?
  .ctrl_read16	      dd ?
  .ctrl_read32	      dd ?

  .ctrl_write8	      dd ?
  .ctrl_write16       dd ?
  .ctrl_write32       dd ?
}

struc CODEC		   ;Audio Chip base class
{
  .chip_id	      dd ?
  .flags	      dd ?
  .status	      dd ?

  .ac_vendor_ids      dd ?    ;ac vendor id string
  .chip_ids	      dd ?    ;chip model string

  .shadow_flag	      dd ?
		      dd ?

  .regs 	      dw ?     ; codec registers
  .reg_master_vol     dw ?     ;0x02
  .reg_aux_out_vol    dw ?     ;0x04
  .reg_mone_vol       dw ?     ;0x06
  .reg_master_tone    dw ?     ;0x08
  .reg_beep_vol       dw ?     ;0x0A
  .reg_phone_vol      dw ?     ;0x0C
  .reg_mic_vol	      dw ?     ;0x0E
  .reg_line_in_vol    dw ?     ;0x10
  .reg_cd_vol	      dw ?     ;0x12
  .reg_video_vol      dw ?     ;0x14
  .reg_aux_in_vol     dw ?     ;0x16
  .reg_pcm_out_vol    dw ?     ;0x18
  .reg_rec_select     dw ?     ;0x1A
  .reg_rec_gain       dw ?     ;0x1C
  .reg_rec_gain_mic   dw ?     ;0x1E
  .reg_gen	      dw ?     ;0x20
  .reg_3d_ctrl	      dw ?     ;0X22
  .reg_page	      dw ?     ;0X24
  .reg_powerdown      dw ?     ;0x26
  .reg_ext_audio      dw ?     ;0x28
  .reg_ext_st	      dw ?     ;0x2a
  .reg_pcm_front_rate dw ?     ;0x2c
  .reg_pcm_surr_rate  dw ?     ;0x2e
  .reg_lfe_rate       dw ?     ;0x30
  .reg_pcm_in_rate    dw ?     ;0x32
		      dw ?     ;0x34
  .reg_cent_lfe_vol   dw ?     ;0x36
  .reg_surr_vol       dw ?     ;0x38
  .reg_spdif_ctrl     dw ?     ;0x3A
		      dw ?     ;0x3C
		      dw ?     ;0x3E
		      dw ?     ;0x40
		      dw ?     ;0x42
		      dw ?     ;0x44
		      dw ?     ;0x46
		      dw ?     ;0x48
		      dw ?     ;0x4A
		      dw ?     ;0x4C
		      dw ?     ;0x4E
		      dw ?     ;0x50
		      dw ?     ;0x52
		      dw ?     ;0x54
		      dw ?     ;0x56
		      dw ?     ;0x58
		      dw ?     ;0x5A
		      dw ?     ;0x5C
		      dw ?     ;0x5E
  .reg_page_0	      dw ?     ;0x60
  .reg_page_1	      dw ?     ;0x62
  .reg_page_2	      dw ?     ;0x64
  .reg_page_3	      dw ?     ;0x66
  .reg_page_4	      dw ?     ;0x68
  .reg_page_5	      dw ?     ;0x6A
  .reg_page_6	      dw ?     ;0x6C
  .reg_page_7	      dw ?     ;0x6E
		      dw ?     ;0x70
		      dw ?     ;0x72
		      dw ?     ;0x74
		      dw ?     ;0x76
		      dw ?     ;0x78
		      dw ?     ;0x7A
  .reg_vendor_id_1    dw ?     ;0x7C
  .reg_vendor_id_2    dw ?     ;0x7E


  .reset	      dd ?    ;virual
  .set_master_vol     dd ?
}

struc CTRL_INFO
{   .pci_cmd	      dd ?
    .irq	      dd ?
    .glob_cntrl       dd ?
    .glob_sta	      dd ?
    .codec_io_base    dd ?
    .ctrl_io_base     dd ?
    .codec_mem_base   dd ?
    .ctrl_mem_base    dd ?
    .codec_id	      dd ?
}

struc IOCTL
{  .handle	      dd ?
   .io_code	      dd ?
   .input	      dd ?
   .inp_size	      dd ?
   .output	      dd ?
   .out_size	      dd ?
}

virtual at 0
  IOCTL IOCTL
end virtual

EVENT_NOTIFY	equ 0x00000200

public START
public service_proc
public version

section '.flat' code readable align 16

proc START stdcall, state:dword

	   cmp	 [state], 1
	   jne	 .stop

     if DEBUG
	   mov	 esi, msgInit
	   call  SysMsgBoardStr
     end if

	   call     detect_controller
	   test     eax, eax
	   jz	    .fail

     if DEBUG
	   mov	 esi,[ctrl.vendor_ids]
	   call  SysMsgBoardStr
	   mov	 esi, [ctrl.ctrl_ids]
	   call  SysMsgBoardStr
     end if

	   call     init_controller
	   test     eax, eax
	   jz	    .fail

	   call     init_codec
	   test     eax, eax
	   jz	    .fail

	   call     setup_codec

	   mov	    esi, msgPrimBuff
	   call     SysMsgBoardStr
	   call     create_primary_buff
	   mov	    esi, msgDone
	   call     SysMsgBoardStr

  if IRQ_REMAP
	   pushf
	   cli

	   mov	    ebx, [ctrl.int_line]
	   in	    al, 0xA1
	   mov	    ah, al
	   in	    al, 0x21
	   test     ebx, ebx
	   jz	    .skip
	   bts	    ax, bx			;mask old line
.skip:
	   bts	    ax, IRQ_LINE		;mask new ine
	   out	    0x21, al
	   mov	    al, ah
	   out	    0xA1, al

	   stdcall  PciWrite8, 0, 0xF8, 0x61, IRQ_LINE	;remap IRQ

	   mov	    dx, 0x4d0			;8259 ELCR1
	   in	    al, dx
	   bts	    ax, IRQ_LINE
	   out	    dx, al			;set level-triggered mode
	   mov	    [ctrl.int_line], IRQ_LINE
	   popf
	   mov	    esi, msgRemap
	   call     SysMsgBoardStr
  end if

	   mov	    eax, VALID_IRQ
	   mov	    ebx, [ctrl.int_line]
	   mov	    esi, msgInvIRQ
	   bt	    eax, ebx
	   jnc	    .fail_msg
	   mov	    eax, ATTCH_IRQ
	   mov	    esi, msgAttchIRQ
	   bt	    eax, ebx
	   jnc	    .fail_msg

	   stdcall  AttachIntHandler, ebx, ac97_irq, dword 0
	   stdcall  emu10k1_create

.reg:
	   stdcall  RegService, sz_sound_srv, service_proc
	   ret
.fail:
     if DEBUG
	   mov	 esi, msgFail
	   call  SysMsgBoardStr
     end if
	   xor	    eax, eax
	   ret
.fail_msg:
	   call     SysMsgBoardStr
	   xor	    eax, eax
	   ret
.stop:
	   call     stop
	   xor	    eax, eax
	   ret
endp

handle	   equ	IOCTL.handle
io_code    equ	IOCTL.io_code
input	   equ	IOCTL.input
inp_size   equ	IOCTL.inp_size
output	   equ	IOCTL.output
out_size   equ	IOCTL.out_size

align 4
proc service_proc stdcall, ioctl:dword

	   mov	    edi, [ioctl]
	   mov	    eax, [edi+io_code]

	   cmp	    eax, SRV_GETVERSION
	   jne	    @F
	   mov	    eax, [edi+output]
	   cmp	    [edi+out_size], 4
	   jne	    .fail

	   mov	    [eax], dword API_VERSION
	   xor	    eax, eax
	   ret
@@:
	   cmp	    eax, DEV_PLAY
	   jne	    @F
     if DEBUG
	   mov	esi, msgPlay
	   call SysMsgBoardStr
     end if
	   call     play
	   ret
@@:
	   cmp	    eax, DEV_STOP
	   jne	    @F
     if DEBUG
	   mov	esi, msgStop
	   call SysMsgBoardStr
     end if
	   call     stop
	   ret
@@:
	   cmp	    eax, DEV_CALLBACK
	   jne	    @F
	   mov	    ebx, [edi+input]
	   stdcall  set_callback, [ebx]
	   ret
@@:
	   cmp	    eax, DEV_SET_MASTERVOL
	   jne	    @F
	   mov	    eax, [edi+input]
	   mov	    eax, [eax]
	   call     set_master_vol	;eax= vol
	   ret
@@:
	   cmp	    eax, DEV_GET_MASTERVOL
	   jne	    @F
	   mov	    ebx, [edi+output]
	   stdcall  get_master_vol, ebx
	   ret
@@:
	   cmp	    eax, DEV_GET_INFO
	   jne	    @F
	   mov	    ebx, [edi+output]
	   stdcall  get_dev_info, ebx
	   ret
@@:
.fail:
	   or	    eax, -1
	   ret
endp

restore   handle
restore   io_code
restore   input
restore   inp_size
restore   output
restore   out_size


align 4
proc ac97_irq
	   locals
	     status dd 0
	   endl

;        status = inl(chip->port + IPR);
	   mov	    edx, IPR
	   call     [ctrl.ctrl_read32]
	   test     eax, eax
	   jz	    @f

	   mov	    dword [status], eax

	   mov	    ebx, dword [buff_list]
	   cmp	    [ctrl.user_callback], 0
	   je	    @f
	   stdcall  [ctrl.user_callback], ebx
       @@:
	   mov	    eax, dword [status] 	  ;; ack ;;
	   mov	    edx, IPR
	   call     [ctrl.ctrl_write32]
	   ret
endp


align 4
proc create_primary_buff

	 ;  Clear page with silence & setup all pointers to this page
	   stdcall  KernelAlloc, 0x10000
	   mov	    [ctrl.buffer], eax

	   mov	    edi, eax
	   mov	    ecx, 0x10000/4
	   xor	    eax, eax
	   cld
	   rep	    stosd

	   mov	    eax, [ctrl.buffer]
	   call     GetPgAddr
	   shl	    eax, 1

	   mov	    edi, pcmout_bdl
	   stosd
	   ;mov      eax, 0x4000000
	   ;stosd

	   mov	    edi, buff_list
	   mov	    eax, [ctrl.buffer]
	   stosd

	   mov	    eax, pcmout_bdl
	   call     GetPgAddr


	;memset(emu->silent_page.area, 0, PAGE_SIZE);
	;silent_page = emu->silent_page.addr << 1;
	;for (idx = 0; idx < MAXPAGES; idx++)
	;        ((u32 *)emu->ptb_pages.area)[idx] = cpu_to_le32(silent_page | idx);
	   stdcall  ptr_write, PTB, 0, eax;emu->ptb_pages.addr
	   stdcall  ptr_write, TCB, 0, 0  ; taken from original driver
	   stdcall  ptr_write, TCBS, 0, 4 ; taken from original driver

	;silent_page = (emu->silent_page.addr << 1) | MAP_PTI_MASK;
	   mov	    eax, [ctrl.buffer]
	   call     GetPgAddr
	   shl	    eax, 1
	   or	    eax, MAP_PTI_MASK
	   xor	    ecx, ecx
	;for (ch = 0; ch < NUM_G; ch++) {
	   stdcall  ptr_write, MAPA, ecx, eax ;ch, silent_page
	   stdcall  ptr_write, MAPB, ecx, eax ;ch, silent_page
	;}

;           stdcall  ptr_write, PLAYBACK_POINTER,   0, 0
;           stdcall  ptr_write, PLAYBACK_UNKNOWN1,  0, 0
;           stdcall  ptr_write, PLAYBACK_UNKNOWN2,  0, 0
;           stdcall  ptr_write, PLAYBACK_DMA_ADDR,  0, eax

;           mov      eax, pcmout_bdl
;           mov      ebx, eax
;           call     GetPgAddr
;           and      ebx, 0xFFF
;           add      eax, ebx

;           stdcall  ptr_write, PLAYBACK_LIST_ADDR, 0, eax
;           stdcall  ptr_write, PLAYBACK_LIST_SIZE, 0, 0
;           stdcall  ptr_write, PLAYBACK_LIST_PTR,  0, 0

	   ;mov     eax, 0x00004000
	   ;shl     eax, 16
;           stdcall  ptr_write, PLAYBACK_PERIOD_SIZE, 0, 0x40000000;eax

	   ret
endp


align 4
proc detect_controller
	   locals
	     last_bus dd ?
	     bus      dd ?
	     devfn    dd ?
	   endl

	   xor eax, eax
	   mov [bus], eax
	   inc eax
	   call PciApi
	   cmp eax, -1
	   je .err

	   mov [last_bus], eax

.next_bus:
	   and [devfn], 0
.next_dev:
	   stdcall PciRead32, [bus], [devfn], dword 0
	   test eax, eax
	   jz .next
	   cmp eax, -1
	   je .next

	   mov edi, devices
@@:
	   mov ebx, [edi]
	   test ebx, ebx
	   jz .next

	   cmp eax, ebx
	   je .found
	   add edi, 12
	   jmp @B
.next:
	   inc [devfn]
	   cmp [devfn], 256
	   jb .next_dev
	   mov eax, [bus]
	   inc eax
	   mov [bus], eax
	   cmp eax, [last_bus]
	   jna .next_bus
	   xor eax, eax
	   ret
.found:
	   mov ebx, [bus]
	   mov [ctrl.bus], ebx

	   mov ecx, [devfn]
	   mov [ctrl.devfn], ecx

	   mov edx, eax
	   and edx, 0xFFFF
	   mov [ctrl.vendor], edx
	   shr eax, 16
	   mov [ctrl.dev_id], eax

	   mov ebx, [edi+4]
	   mov [ctrl.ctrl_ids], ebx
	   mov esi, [edi+8]
	   mov [ctrl.ctrl_setup], esi

	   cmp edx, VID_Creative
	   jne @F
	   mov [ctrl.vendor_ids], msg_Creative
	   ret
@@:

.err:
	   xor eax, eax
	   mov [ctrl.vendor_ids], eax	  ;something  wrong ?
	   ret
endp

align 4
proc init_controller

	   stdcall  PciRead32, [ctrl.bus], [ctrl.devfn], dword 0x2C
	   mov	    dword [ctrl_serial], eax

	   mov	    esi, msgPciSubsys
	   call     SysMsgBoardStr
	   call     dword2str
	   call     SysMsgBoardStr

	   stdcall  PciRead8, [ctrl.bus], [ctrl.devfn], dword 8
	   mov	    byte [ctrl_rev], al
	   and	    eax, 0xFF

	   mov	    esi, msgPciRev
	   call     SysMsgBoardStr
	   call     dword2str
	   call     SysMsgBoardStr

	   stdcall  PciRead32, [ctrl.bus], [ctrl.devfn], dword 4
	   mov	    ebx, eax
	   and	    eax, 0xFFFF
	   mov	    [ctrl.pci_cmd], eax
	   shr	    ebx, 16
	   mov	    [ctrl.pci_stat], ebx

	   mov	    esi, msgPciCmd
	   call     SysMsgBoardStr
	   call     dword2str
	   call     SysMsgBoardStr

	   mov	    esi, msgPciStat
	   call     SysMsgBoardStr
	   mov	    eax, [ctrl.pci_stat]
	   call     dword2str
	   call     SysMsgBoardStr

	   mov	    esi, msgCtrlIsaIo
	   call     SysMsgBoardStr
	   stdcall  PciRead32, [ctrl.bus], [ctrl.devfn], dword 0x10
	   call     dword2str
	   call     SysMsgBoardStr

	   and	    eax, 0xFFC0
	   mov	    [ctrl.ctrl_io_base], eax

.default:
	   stdcall  PciRead32, [ctrl.bus], [ctrl.devfn], dword 0x3C
	   and	    eax, 0xFF
@@:
	   mov	    [ctrl.int_line], eax

	   call     [ctrl.ctrl_setup]

	   cmp	    [ctrl.dev_id], 0x0004 ; Audigy ?    - FIX THIS.
	   jne	    .not_audigy
	   mov	    dword [ptr_addr_mask], A_PTR_ADDRESS_MASK
     .not_audigy:

	   xor	    eax, eax
	   inc	    eax
	   ret
endp

align 4
proc set_Creative
	   mov	    [ctrl.codec_read16],  codec_io_r16	  ;virtual
	   mov	    [ctrl.codec_write16], codec_io_w16	  ;virtual

	   mov	    [ctrl.ctrl_read8 ],  ctrl_io_r8	  ;virtual
	   mov	    [ctrl.ctrl_read16],  ctrl_io_r16	  ;virtual
	   mov	    [ctrl.ctrl_read32],  ctrl_io_r32	  ;virtual

	   mov	    [ctrl.ctrl_write8 ], ctrl_io_w8	  ;virtual
	   mov	    [ctrl.ctrl_write16], ctrl_io_w16	  ;virtual
	   mov	    [ctrl.ctrl_write32], ctrl_io_w32	  ;virtual
	   ret
endp


align 4
proc init_codec
	   call     reset_codec
	   test     eax, eax
	   jz	    .err
	   call     detect_codec
	   xor	    eax, eax
	   inc	    eax
	   ret
     .err:
	   xor	    eax, eax
	   ret
endp

align 4
proc reset_codec
	   locals
	     counter dd ?
	   endl

     if DEBUG
	   mov	 esi, msgCold
	   call  SysMsgBoardStr
     end if

	   mov	    eax, 100000     ; wait 100 ms ;400000     ; wait 400 ms
	   call     StallExec

;           stdcall  ptr_read, TRIGGER_CHANNEL, 0

	   mov	    [counter], 16    ; total 20*100 ms = 2s
.wait:
	   stdcall  codec_read, dword 0x26
	   test     eax, 1
	   jnz	    .ok

	   mov	    eax, 100000    ; wait 100 ms
	   call     StallExec

	   dec	    [counter]
	   jnz	    .wait

     if DEBUG
	   mov	 esi, msgCRFail
	   call  SysMsgBoardStr
     end if

.fail:
	   stc
	   ret
.ok:


	   xor	    eax, eax
	   inc	    eax
	   ret
endp


align 4
play:
;           mov      eax, INTE_CH_0_LOOP
;           stdcall  intr_enable, eax

;           stdcall  ptr_read, TRIGGER_CHANNEL, 0
;           mov      ebx, TRIGGER_CHANNEL_0
;           or       eax, ebx
;           stdcall  ptr_write, TRIGGER_CHANNEL, 0, eax

	   xor	    eax, eax
	   ret

align 4
stop:
;           mov      eax, INTE_CH_0_LOOP or INTE_CH_0_HALF_LOOP
;           stdcall  intr_disable, eax

;           stdcall  ptr_read, TRIGGER_CHANNEL, 0
;           mov      ebx, TRIGGER_CHANNEL_0
;           xor      ebx, -1
;           or       eax, ebx
;           stdcall  ptr_write, TRIGGER_CHANNEL, 0, eax
	   xor	    eax, eax
	   ret

align 4
proc get_dev_info stdcall, p_info:dword
	   virtual at esi
	     CTRL_INFO CTRL_INFO
	   end virtual

	   mov	    esi, [p_info]
	   mov	    eax, [ctrl.int_line]
	   mov	    ecx, [ctrl.ctrl_io_base]
	   mov	    [CTRL_INFO.irq], eax
	   mov	    [CTRL_INFO.ctrl_io_base], ecx
	   mov	    eax, [codec.chip_id]
	   mov	    [CTRL_INFO.codec_id], eax
	   mov	    ebx, [ctrl.pci_cmd]
	   mov	    [CTRL_INFO.pci_cmd], ebx

	   xor	    eax, eax
	   mov	    [CTRL_INFO.codec_io_base], eax
	   mov	    [CTRL_INFO.codec_mem_base], eax
	   mov	    [CTRL_INFO.ctrl_mem_base], eax
	   mov	    [CTRL_INFO.glob_cntrl], eax
	   mov	    [CTRL_INFO.glob_sta], eax
	   ret
endp

align 4
proc set_callback stdcall, handler:dword
	   mov	    eax, [handler]
	   mov	    [ctrl.user_callback], eax
	   ret
endp


align 4
proc emu10k1_probe stdcall
	   stdcall  emu10k1_create
;           stdcall  emu10k1_init_partII
	stdcall  emu10k1_pcm, 0, 0
	test  eax, eax
	jz    @f
	ret
     @@:
;        stdcall  emu10k1_pcm_mic, 1, 0
;        test  eax, eax
;        jz    @f
;        ret
;     @@:
;        stdcall  emu10k1_pcm_efx, 2, 0
;        test  eax, eax
;        jz    @f
;        ret
;     @@:
	; This stores the periods table.
;        if (emu->audigy && emu->revision == 4) { /* P16V */
;                if(snd_dma_alloc_pages(SNDRV_DMA_TYPE_DEV, snd_dma_pci_data(pci), 1024, &emu->p16v_buffer) < 0) {
;                        snd_p16v_free(emu);
;                        return -ENOMEM;
;                }
;        }

	stdcall  emu10k1_mixer
	test  eax, eax
	jz    @f
	ret
     @@:
;        stdcall  emu10k1_timer, 0
;        test  eax, eax
;        jz    @f
;        ret
;     @@:
;        stdcall  emu10k1_pcm_multi, 3, 0
;        test  eax, eax
;        jz    @f
;        ret
;     @@:
;        if (emu->audigy && emu->revision == 4) { /* P16V */
;                if ((err = snd_p16v_pcm(emu, 4, NULL)) < 0) {
;                 test  eax, eax
;                 jz    @f
;                 ret
;               @@:
;        }
;        if (emu->audigy) {
;                 stdcall  emu10k1_audigy_midi
;                 test  eax, eax
;                 jz    @f
;                 ret
;               @@:
;        } else {
;                stdcall  emu10k1_midi
;                test  eax, eax
;                jz    @f
;                ret
;              @@:
;        }
;        stdcall  emu10k1_fx8010_new, 0, 0
;        test  eax, eax
;        jz    @f
;        ret
;     @@:
;#ifdef ENABLE_SYNTH ... #endif
	   ret
endp


;*************************************************************************;
;* EMU10K1 init / done                                                   *;
;*************************************************************************;

align 4
proc  voice_init stdcall, ch:dword
	stdcall  ptr_write, DCYSUSV, [ch], 0
	stdcall  ptr_write, IP, [ch], 0
	stdcall  ptr_write, VTFT, [ch], 0xFFFF
	stdcall  ptr_write, CVCF, [ch], 0xFFFF
	stdcall  ptr_write, PTRX, [ch], 0
	stdcall  ptr_write, CPF, [ch], 0
	stdcall  ptr_write, CCR, [ch], 0

	stdcall  ptr_write, PSST, [ch], 0
	stdcall  ptr_write, DSL, [ch], 0x10
	stdcall  ptr_write, CCCA, [ch], 0
	stdcall  ptr_write, Z1, [ch], 0
	stdcall  ptr_write, Z2, [ch], 0
	stdcall  ptr_write, FXRT, [ch], 0x32100000

	stdcall  ptr_write, ATKHLDM, [ch], 0
	stdcall  ptr_write, DCYSUSM, [ch], 0
	stdcall  ptr_write, IFATN, [ch], 0xFFFF
	stdcall  ptr_write, PEFE, [ch], 0
	stdcall  ptr_write, FMMOD, [ch], 0
	stdcall  ptr_write, TREMFRQ, [ch], 24	 ; 1 Hz
	stdcall  ptr_write, FM2FRQ2, [ch], 24	 ; 1 Hz
	stdcall  ptr_write, TEMPENV, [ch], 0

	; these are last so OFF prevents writing
	stdcall  ptr_write, LFOVAL2, [ch], 0
	stdcall  ptr_write, LFOVAL1, [ch], 0
	stdcall  ptr_write, ATKHLDV, [ch], 0
	stdcall  ptr_write, ENVVOL, [ch], 0
	stdcall  ptr_write, ENVVAL, [ch], 0

	; Audigy extra stuffs
	cmp	 dword [ptr_addr_mask], A_PTR_ADDRESS_MASK ; Audigy ?
	jne	 .not_audigy
		stdcall  ptr_write, 0x4C, [ch], 0 ; ??
		stdcall  ptr_write, 0x4D, [ch], 0 ; ??
		stdcall  ptr_write, 0x4E, [ch], 0 ; ??
		stdcall  ptr_write, 0x4F, [ch], 0 ; ??
		stdcall  ptr_write, A_FXRT1, [ch], 0x03020100
		stdcall  ptr_write, A_FXRT2, [ch], 0x3F3F3F3F
		stdcall  ptr_write, A_SENDAMOUNTS, [ch], 0
	.not_audigy:
endp


align 4
proc emu10k1_create stdcall
	   stdcall  PciRead16, [ctrl.bus], [ctrl.devfn], dword 4
	   test     eax, 4 ; test master bit
	   jnz	    @f
	   or	    eax, 4
	   stdcall  PciWrite16, [ctrl.bus], [ctrl.devfn], dword 4, eax ; set master bit
	 @@:
	   stdcall  emu10k1_init_partI
	   stdcall  emu10k1_init_partII
	   ret
endp

align 4
proc emu10k1_init_partI stdcall
	; disable audio and lock cache
	   mov	    eax, HCFG_LOCKSOUNDCACHE or HCFG_LOCKTANKCACHE_MASK or HCFG_MUTEBUTTONENABLE
	   mov	    edx, HCFG
	   call     [ctrl.ctrl_write32]
	; reset recording buffers
	   stdcall  ptr_write, MICBS, 0, ADCBS_BUFSIZE_NONE
	   stdcall  ptr_write, MICBA, 0, 0
	   stdcall  ptr_write, FXBS, 0, ADCBS_BUFSIZE_NONE
	   stdcall  ptr_write, FXBA, 0, 0
	   stdcall  ptr_write, ADCBS, 0, ADCBS_BUFSIZE_NONE
	   stdcall  ptr_write, ADCBA, 0, 0
	; disable channel interrupt
	   xor	    eax, eax
	   mov	    edx, INTE
	   call     [ctrl.ctrl_write32]

	   stdcall  ptr_write, CLIEL, 0, 0
	   stdcall  ptr_write, CLIEH, 0, 0
	   stdcall  ptr_write, SOLEL, 0, 0
	   stdcall  ptr_write, SOLEH, 0, 0

	   cmp	    dword [ptr_addr_mask], A_PTR_ADDRESS_MASK ; Audigy?
	   jne	    @f
	   ; set SPDIF bypass mode
	   stdcall  ptr_write, SPBYPASS, 0, SPBYPASS_FORMAT
	   ; enable rear left + rear right AC97 slots
	   stdcall  ptr_write, AC97SLOT, 0, AC97SLOT_REAR_RIGHT or AC97SLOT_REAR_LEFT
     @@:

	; init envelope engine
	   ;for (ch = 0; ch < NUM_G; ch++) {
	   ;     emu->voices[ch].emu = emu;
	   ;     emu->voices[ch].number = ch;
		stdcall voice_init, 0;ch
	   ;}

	   stdcall  ptr_write, SPCS0, 0, \
			       SPCS_CLKACCY_1000PPM or SPCS_SAMPLERATE_48 or \
			       SPCS_CHANNELNUM_LEFT or SPCS_SOURCENUM_UNSPEC or \
			       SPCS_GENERATIONSTATUS or 0x00001200 or \
			       0x00000000 or SPCS_EMPHASIS_NONE or SPCS_COPYRIGHT
	   stdcall  ptr_write, SPCS1, 0, \
			       SPCS_CLKACCY_1000PPM or SPCS_SAMPLERATE_48 or \
			       SPCS_CHANNELNUM_LEFT or SPCS_SOURCENUM_UNSPEC or \
			       SPCS_GENERATIONSTATUS or 0x00001200 or \
			       0x00000000 or SPCS_EMPHASIS_NONE or SPCS_COPYRIGHT
	   stdcall  ptr_write, SPCS2, 0, \
			       SPCS_CLKACCY_1000PPM or SPCS_SAMPLERATE_48 or \
			       SPCS_CHANNELNUM_LEFT or SPCS_SOURCENUM_UNSPEC or \
			       SPCS_GENERATIONSTATUS or 0x00001200 or \
			       0x00000000 or SPCS_EMPHASIS_NONE or SPCS_COPYRIGHT

	;if (emu->card_capabilities->ca0151_chip) { /* audigy2 */ ...}
	;if (emu->audigy && (emu->serial == 0x10011102) ) { /* audigy2 Value */ ...}




;           mov      eax, dword HCFG_LOCKSOUNDCACHE or HCFG_AUDIOENABLE
;           mov      edx, HCFG
;           call     [ctrl.ctrl_write32]

	   ret
endp

align 4
proc emu10k1_init_partII stdcall
	;;
	;;  Hokay, setup HCFG
	;;   Mute Disable Audio = 0
	;;   Lock Tank Memory = 1
	;;   Lock Sound Memory = 0
	;;   Auto Mute = 1
	;;
	   cmp	    dword [ptr_addr_mask], A_PTR_ADDRESS_MASK ; Audigy ?
	   jne	    .1not_audigy

	   cmp	    byte [ctrl_rev], 4	; Audigy2 ?
	   jne	    @f

	   mov	    eax, HCFG_AUDIOENABLE or HCFG_AC3ENABLE_CDSPDIF or HCFG_AC3ENABLE_GPSPDIF \
			  or HCFG_AUTOMUTE or HCFG_JOYENABLE
	   jmp	    .write_HCFG
     @@:
	   mov	    eax, HCFG_AUTOMUTE or HCFG_JOYENABLE
	   jmp	    .write_HCFG
     .1not_audigy:
	   mov	    eax, HCFG_LOCKTANKCACHE_MASK or HCFG_AUTOMUTE
     .write_HCFG:
	   mov	    edx, HCFG
	   call     [ctrl.ctrl_write32]

	;if (enable_ir) {        /* enable IR for SB Live */
	;        if (emu->audigy) {
	   cmp	    dword [ptr_addr_mask], A_PTR_ADDRESS_MASK ; Audigy ?
	   jne	    .2not_audigy
			;unsigned int reg = inl(emu->port + A_IOCFG);
			;outl(reg | A_IOCFG_GPOUT2, emu->port + A_IOCFG);
			;udelay(500);
			;outl(reg | A_IOCFG_GPOUT1 | A_IOCFG_GPOUT2, emu->port + A_IOCFG);
			;udelay(100);
			;outl(reg, emu->port + A_IOCFG);
	   mov	    edx, A_IOCFG
	   call     [ctrl.ctrl_read32]
	   mov	    ebx, eax
	   or	    eax, A_IOCFG_GPOUT2
	   call     [ctrl.ctrl_write32]
	   mov	    eax, 500000   ; wait 500 ms
	   call     StallExec
	   or	    eax, A_IOCFG_GPOUT1
	   call     [ctrl.ctrl_write32]
	   mov	    eax, 100000   ; wait 100 ms
	   call     StallExec
	   mov	    eax, ebx
	   call     [ctrl.ctrl_write32]
	   jmp	    @f
     .2not_audigy:
	;        } else {
			;unsigned int reg = inl(emu->port + HCFG);
			;outl(reg | HCFG_GPOUT2, emu->port + HCFG);
			;udelay(500);
			;outl(reg | HCFG_GPOUT1 | HCFG_GPOUT2, emu->port + HCFG);
			;udelay(100);
			;outl(reg, emu->port + HCFG);
       ;         }
       ; }
	   mov	    edx, HCFG
	   call     [ctrl.ctrl_read32]
	   mov	    ebx, eax
	   or	    eax, HCFG_GPOUT2
	   call     [ctrl.ctrl_write32]
	   mov	    eax, 500000   ; wait 500 ms
	   call     StallExec
	   or	    eax, HCFG_GPOUT1
	   call     [ctrl.ctrl_write32]
	   mov	    eax, 100000   ; wait 100 ms
	   call     StallExec
	   mov	    eax, ebx
	   call     [ctrl.ctrl_write32]
     @@:
	;if (emu->audigy) {      /* enable analog output */
	   cmp	    dword [ptr_addr_mask], A_PTR_ADDRESS_MASK ; Audigy ?
	   jne	    .3not_audigy

		;unsigned int reg = inl(emu->port + A_IOCFG);
		;outl(reg | A_IOCFG_GPOUT0, emu->port + A_IOCFG);
	   mov	    edx, A_IOCFG
	   call     [ctrl.ctrl_read32]
	   or	    eax, A_IOCFG_GPOUT0
	   call     [ctrl.ctrl_write32]
	;}
     .3not_audigy:

	 ;  Initialize the effect engine
	;if ((err = snd_emu10k1_init_efx(emu)) < 0)
	;        return err;


	 ;  Enable the audio bit
	  ;outl(inl(emu->port + HCFG) | HCFG_AUDIOENABLE, emu->port + HCFG);
	   mov	    edx, HCFG
	   call     [ctrl.ctrl_read32]
	   or	    eax, HCFG_AUDIOENABLE
	   call     [ctrl.ctrl_write32]

	 ; Enable analog/digital outs on audigy
	   ;if (emu->audigy) {
	   cmp	    dword [ptr_addr_mask], A_PTR_ADDRESS_MASK ; Audigy ?
	   jne	    .4not_audigy
		;outl(inl(emu->port + A_IOCFG) & ~0x44, emu->port + A_IOCFG);
	   mov	    edx, A_IOCFG
	   call     [ctrl.ctrl_read32]
	   mov	    ebx, 0x44
	   xor	    ebx, -1
	   and	    eax, ebx
	   call     [ctrl.ctrl_write32]

		;if (emu->revision == 4) { /* audigy2 */
	   cmp	    byte [ctrl_rev], 4	; Audigy2 ?
	   jne	    @f
			;; Unmute Analog now.  Set GPO6 to 1 for Apollo.
			;; This has to be done after init ALice3 I2SOut beyond 48KHz.
			;; So, sequence is important.
			;outl(inl(emu->port + A_IOCFG) | 0x0040, emu->port + A_IOCFG);
	   mov	    edx, A_IOCFG
	   call     [ctrl.ctrl_read32]
	   or	    eax, 0x40
	   jmp	    .write_A_IOCFG
     @@:
		;} else if (emu->serial == 0x10011102) { /* audigy2 value */
	   cmp	    dword [ctrl_serial], 0x10011102  ; Audigy2 value
	   jne	    @f
			;; Unmute Analog now.
			;outl(inl(emu->port + A_IOCFG) | 0x0060, emu->port + A_IOCFG);
	   mov	    edx, A_IOCFG
	   call     [ctrl.ctrl_read32]
	   or	    eax, 0x60
	   jmp	    .write_A_IOCFG
     @@:
		;} else {
			;; Disable routing from AC97 line out to Front speakers
			;outl(inl(emu->port + A_IOCFG) | 0x0080, emu->port + A_IOCFG);
	   mov	    edx, A_IOCFG
	   call     [ctrl.ctrl_read32]
	   or	    eax, 0x80
		;}
	   ;}
     .write_A_IOCFG:
	   call     [ctrl.ctrl_write32]
     .4not_audigy:
	
	   ;#if 0 ... #endif

	   stdcall  intr_enable, INTE_PCIERRORENABLE

	;emu->reserved_page = (emu10k1_memblk_t *)snd_emu10k1_synth_alloc(emu, 4096);
	;if (emu->reserved_page) emu->reserved_page->map_locked = 1;
	   ret
endp


align 4
proc codec_read stdcall, reg:dword  ;+
	   stdcall  ac97_read, dword [reg]
	   ret
endp

align 4
proc codec_write stdcall, reg:dword  ;+
	   stdcall  ac97_write, dword [reg], eax
	   ret
endp




align 4
proc StallExec
	   push     ecx
	   push     edx
	   push     ebx
	   push     eax

	   mov	    ecx, CPU_FREQ
	   mul	    ecx
	   mov	    ebx, eax	   ;low
	   mov	    ecx, edx	   ;high
	   rdtsc
	   add	    ebx, eax
	   adc	    ecx, edx
@@:
	   rdtsc
	   sub	    eax, ebx
	   sbb	    edx, ecx
	   js	    @B

	   pop	    eax
	   pop	    ebx
	   pop	    edx
	   pop	    ecx
	   ret
endp

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;          CONTROLLER IO functions
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

align 4
proc codec_io_r16 ;Not used.
	   ;mov      edx, [ctrl.ctrl_io_base]
	   ;in       eax, dx
	   ret
endp

align 4
proc codec_io_w16 ;Not used.
	   ;mov      edx, [ctrl.ctrl_io_base]
	   ;out      dx,  eax
	   ret
endp

align 4
proc ctrl_io_r8
	   add	    edx, [ctrl.ctrl_io_base]
	   in	    al, dx
	   ret
endp

align 4
proc ctrl_io_r16
	   add	    edx, [ctrl.ctrl_io_base]
	   in	    ax, dx
	   ret
endp

align 4
proc ctrl_io_r32
	   add	    edx, [ctrl.ctrl_io_base]
	   in	    eax, dx
	   ret
endp

align 4
proc ctrl_io_w8
	   add	    edx, [ctrl.ctrl_io_base]
	   out	    dx, al
	   ret
endp

align 4
proc ctrl_io_w16
	   add	    edx, [ctrl.ctrl_io_base]
	   out	    dx, ax
	   ret
endp

align 4
proc ctrl_io_w32
	   add	    edx, [ctrl.ctrl_io_base]
	   out	    dx, eax
	   ret
endp


align 4
dword2str:
	   push     eax ebx ecx
	   mov	    esi, hex_buff
	   mov	    ecx, -8
   @@:
	   rol	    eax, 4
	   mov	    ebx, eax
	   and	    ebx, 0x0F
	   mov	    bl, [ebx+hexletters]
	   mov	    [8+esi+ecx], bl
	   inc	    ecx
	   jnz	    @B
	   pop	    ecx ebx eax
	   ret

hexletters   db '0123456789ABCDEF'
hex_buff     db 8 dup(0),13,10,0


include "codec.inc"

align 4
devices dd (CTRL_EMU10K1 shl 16)+VID_Creative,msg_EMU10K1,set_Creative
	dd (CTRL_Audigy  shl 16)+VID_Creative,msg_Audigy, set_Creative
	dd (CTRL_Audigy2 shl 16)+VID_Creative,msg_Audigy2,set_Creative
	dd 0	;terminator


version      dd (5 shl 16) or (API_VERSION and 0xFFFF)

msg_EMU10K1	 db 'SB Live!', 13,10, 0
msg_Audigy	 db 'SB Audigy', 13,10, 0
msg_Audigy2	 db 'SB Audigy2', 13,10, 0

msg_Creative	 db 'Creative ', 0

szKernel	    db 'KERNEL', 0
sz_sound_srv	    db 'SOUND',0

msgInit      db 'detect hardware...',13,10,0
msgFail      db 'device not found',13,10,0
msgAttchIRQ  db 'IRQ line not supported', 13,10, 0
msgInvIRQ    db 'IRQ line not assigned or invalid', 13,10, 0
msgPlay      db 'start play', 13,10,0
msgStop      db 'stop play',  13,10,0
msgIRQ	     db 'AC97 IRQ', 13,10,0
;msgInitCtrl  db 'init controller',13,10,0
;msgInitCodec db 'init codec',13,10,0
msgPrimBuff  db 'create primary buffer ...',0
msgDone      db 'done',13,10,0
msgRemap     db 'Remap IRQ',13,10,0
;msgReg       db 'set service handler',13,10,0
;msgOk        db 'service installed',13,10,0
msgCold      db 'cold reset',13,10,0
;msgWarm      db 'warm reset',13,10,0
;msgWRFail    db 'warm reset failed',13,10,0
msgCRFail    db 'cold reset failed',13,10,0
;msgCFail     db 'codec not ready',13,10,0
;msgCInvalid  db 'codec is not valid',13,10,0 ;Asper
;msgResetOk   db 'reset complete',13,10,0
;msgStatus    db 'global status   ',0
;msgControl   db 'global control  ',0
msgPciCmd    db 'PCI command     ',0
msgPciStat   db 'PCI status      ',0
msgPciSubsys db 'PCI subsystem   ',0
msgPciRev    db 'PCI revision    ',0
msgCtrlIsaIo db 'controller io base   ',0
;msgMixIsaIo  db 'codec io base        ',0
;msgCtrlMMIo  db 'controller mmio base ',0
;msgMixMMIo   db 'codec mmio base      ',0
;msgIrqMap    db 'AC97 irq map as      ',0

ptr_addr_mask	 dd PTR_ADDRESS_MASK

section '.data' data readable writable align 16

pcmout_bdl	 rq 32
buff_list	 rd 32

ctrl_rev	 rb  1
ctrl_serial	 rd  1

codec CODEC
ctrl AC_CNTRL
