;; ------------------- DEFINES -------------------- ;;

;Asper [
;Chip type bits
AC97_CHIP		equ	0000000000000001b ;1
SPK71_CHIP		equ	0000000000000010b ;2

EMU10K1_CHIP		equ	0000000000000100b ;4
SBLIVE51_CHIP		equ	0000000000001000b ;8

AUDIGY_CHIP		equ	0000000000010000b ;16
CA0102_CHIP		equ	0000000000100000b ;32
CA0108_CHIP		equ	0000000001000000b ;64
CA0151_CHIP		equ	0000000010000000b ;128
CA_CARDBUS_CHIP 	equ	0000000100000000b ;256

SPI_DAC_CHIP		equ	0000001000000000b ;512
I2C_ADC_CHIP		equ	0000010000000000b ;1024
ECARD_CHIP		equ	0000100000000000b ;2048
;ADC_1361T_CHIP         equ     0001000000000000b ;4096

EMU_MODEL_EMU1616	equ	0010000000000000b ;8192
EMU_MODEL_EMU1010	equ	0100000000000000b ;16384
EMU_MODEL_EMU1010B	equ	1000000000000000b ;32768
EMU_MODEL_EMU0404	equ    10000000000000000b ;65536
EMU_MODEL_CHIP		equ	EMU_MODEL_EMU1616 or EMU_MODEL_EMU1010 or EMU_MODEL_EMU1010B or EMU_MODEL_EMU0404

;SPDIF_BUG              equ   100000000000000000b ;131072
;INVERT_SHARED_SPDIF    equ  1000000000000000000b ;262144

;]



EMUPAGESIZE	    equ 4096
MAXREQVOICES	    equ 8
MAXPAGES	    equ 8192
RESERVED	    equ 0
NUM_MIDI	    equ 16
NUM_G		    equ 64		;; use all channels ;;
NUM_FXSENDS	    equ 4
NUM_EFX_PLAYBACK    equ 16

;; FIXME? - according to the OSS driver the EMU10K1 needs a 29 bit DMA mask ;;
EMU10K1_DMA_MASK	equ 0x7FFFFFFF	  ;; 31bit ;;
AUDIGY_DMA_MASK 	equ 0xFFFFFFFF	  ;; 32bit ;;

TMEMSIZE	equ 256*1024
TMEMSIZEREG	equ 4

;; Audigy specify registers are prefixed with 'A_'

;EMU10K1 host controller registers set
;; common offsets
;; some definitions were borrowed from emu10k1 driver as they seem to be the same
;;**********************************************************************************************;;
;; PCI function 0 registers, address = <val> + PCIBASE0                                         ;;
;;**********************************************************************************************;;

PTR			equ 0x00	    ;; Indexed register set pointer register        ;;
						;; NOTE: The CHANNELNUM and ADDRESS words can   ;;
						;; be modified independently of each other.     ;;
PTR_CHANNELNUM_MASK	equ 0x0000003F	    ;; For each per-channel register, indicates the ;;
						;; channel number of the register to be         ;;
						;; accessed.  For non per-channel registers the ;;
						;; value should be set to zero.                 ;;
PTR_ADDRESS_MASK	equ 0x07FF0000	    ;; Register index                               ;;
A_PTR_ADDRESS_MASK	equ 0x0FFF0000

DATA			equ 0x04	    ;; Indexed register set data register           ;;

IPR			equ 0x08	    ;; Global interrupt pending register            ;;
						;; Clear pending interrupts by writing a 1 to   ;;
						;; the relevant bits and zero to the other bits ;;
IPR_P16V		equ 0x80000000	    ;; Bit set when the CA0151 P16V chip wishes to interrupt ;;
IPR_GPIOMSG		equ 0x20000000	    ;; GPIO message interrupt (RE'd, still not sure
						;;   which INTE bits enable it)                 ;;

;; The next two interrupts are for the midi port on the Audigy Drive (A_MPU1)                   ;;
IPR_A_MIDITRANSBUFEMPTY2 equ 0x10000000     ;; MIDI UART transmit buffer empty              ;;
IPR_A_MIDIRECVBUFEMPTY2  equ 0x08000000     ;; MIDI UART receive buffer empty               ;;

IPR_SPDIFBUFFULL	equ 0x04000000	    ;; SPDIF capture related, 10k2 only? (RE)       ;;
IPR_SPDIFBUFHALFFULL	equ 0x02000000	    ;; SPDIF capture related? (RE)                  ;;

IPR_SAMPLERATETRACKER	equ 0x01000000	    ;; Sample rate tracker lock status change       ;;
IPR_FXDSP		equ 0x00800000	    ;; Enable FX DSP interrupts                     ;;
IPR_FORCEINT		equ 0x00400000	    ;; Force Sound Blaster interrupt                ;;
IPR_PCIERROR		equ 0x00200000	    ;; PCI bus error                                ;;
IPR_VOLINCR		equ 0x00100000	    ;; Volume increment button pressed              ;;
IPR_VOLDECR		equ 0x00080000	    ;; Volume decrement button pressed              ;;
IPR_MUTE		equ 0x00040000	    ;; Mute button pressed                          ;;
IPR_MICBUFFULL		equ 0x00020000	    ;; Microphone buffer full                       ;;
IPR_MICBUFHALFFULL	equ 0x00010000	    ;; Microphone buffer half full                  ;;
IPR_ADCBUFFULL		equ 0x00008000	    ;; ADC buffer full                              ;;
IPR_ADCBUFHALFFULL	equ 0x00004000	    ;; ADC buffer half full                         ;;
IPR_EFXBUFFULL		equ 0x00002000	    ;; Effects buffer full                          ;;
IPR_EFXBUFHALFFULL	equ 0x00001000	    ;; Effects buffer half full                     ;;
IPR_GPSPDIFSTATUSCHANGE equ 0x00000800	    ;; GPSPDIF channel status change                ;;
IPR_CDROMSTATUSCHANGE	equ 0x00000400	    ;; CD-ROM channel status change                 ;;
IPR_INTERVALTIMER	equ 0x00000200	    ;; Interval timer terminal count                ;;
IPR_MIDITRANSBUFEMPTY	equ 0x00000100	    ;; MIDI UART transmit buffer empty              ;;
IPR_MIDIRECVBUFEMPTY	equ 0x00000080	    ;; MIDI UART receive buffer empty               ;;
IPR_CHANNELLOOP 	equ 0x00000040	    ;; Channel (half) loop interrupt(s) pending     ;;
IPR_CHANNELNUMBERMASK	equ 0x0000003F	    ;; When IPR_CHANNELLOOP is set, indicates the   ;;
						;; highest set channel in CLIPL, CLIPH, HLIPL,  ;;
						;; or HLIPH.  When IP is written with CL set,   ;;
						;; the bit in H/CLIPL or H/CLIPH corresponding  ;;
						;; to the CIN value written will be cleared.    ;;

INTE			equ 0x0C	    ;; Interrupt enable register                    ;;
INTE_VIRTUALSB_MASK	equ 0xC0000000	    ;; Virtual Soundblaster I/O port capture        ;;
INTE_VIRTUALSB_220	equ 0x00000000	    ;; Capture at I/O base address 0x220-0x22f      ;;
INTE_VIRTUALSB_240	equ 0x40000000	    ;; Capture at I/O base address 0x240            ;;
INTE_VIRTUALSB_260	equ 0x80000000	    ;; Capture at I/O base address 0x260            ;;
INTE_VIRTUALSB_280	equ 0xC0000000	    ;; Capture at I/O base address 0x280            ;;
INTE_VIRTUALMPU_MASK	equ 0x30000000	    ;; Virtual MPU I/O port capture                 ;;
INTE_VIRTUALMPU_300	equ 0x00000000	    ;; Capture at I/O base address 0x300-0x301      ;;
INTE_VIRTUALMPU_310	equ 0x10000000	    ;; Capture at I/O base address 0x310            ;;
INTE_VIRTUALMPU_320	equ 0x20000000	    ;; Capture at I/O base address 0x320            ;;
INTE_VIRTUALMPU_330	equ 0x30000000	    ;; Capture at I/O base address 0x330            ;;
INTE_MASTERDMAENABLE	equ 0x08000000	    ;; Master DMA emulation at 0x000-0x00f          ;;
INTE_SLAVEDMAENABLE	equ 0x04000000	    ;; Slave DMA emulation at 0x0c0-0x0df           ;;
INTE_MASTERPICENABLE	equ 0x02000000	    ;; Master PIC emulation at 0x020-0x021          ;;
INTE_SLAVEPICENABLE	equ 0x01000000	    ;; Slave PIC emulation at 0x0a0-0x0a1           ;;
INTE_VSBENABLE		equ 0x00800000	    ;; Enable virtual Soundblaster                  ;;
INTE_ADLIBENABLE	equ 0x00400000	    ;; Enable AdLib emulation at 0x388-0x38b        ;;
INTE_MPUENABLE		equ 0x00200000	    ;; Enable virtual MPU                           ;;
INTE_FORCEINT		equ 0x00100000	    ;; Continuously assert INTAN                    ;;

INTE_MRHANDENABLE	equ 0x00080000	    ;; Enable the "Mr. Hand" logic                  ;;
						;; NOTE: There is no reason to use this under   ;;
						;; Linux, and it will cause odd hardware        ;;
						;; behavior and possibly random segfaults and   ;;
						;; lockups if enabled.                          ;;

;; The next two interrupts are for the midi port on the Audigy Drive (A_MPU1)                   ;;
INTE_A_MIDITXENABLE2	equ 0x00020000	    ;; Enable MIDI transmit-buffer-empty interrupts ;;
INTE_A_MIDIRXENABLE2	equ 0x00010000	    ;; Enable MIDI receive-buffer-empty interrupts  ;;


INTE_SAMPLERATETRACKER	equ 0x00002000	    ;; Enable sample rate tracker interrupts        ;;
						;; NOTE: This bit must always be enabled        ;;
INTE_FXDSPENABLE	equ 0x00001000	    ;; Enable FX DSP interrupts                     ;;
INTE_PCIERRORENABLE	equ 0x00000800	    ;; Enable PCI bus error interrupts              ;;
INTE_VOLINCRENABLE	equ 0x00000400	    ;; Enable volume increment button interrupts    ;;
INTE_VOLDECRENABLE	equ 0x00000200	    ;; Enable volume decrement button interrupts    ;;
INTE_MUTEENABLE 	equ 0x00000100	    ;; Enable mute button interrupts                ;;
INTE_MICBUFENABLE	equ 0x00000080	    ;; Enable microphone buffer interrupts          ;;
INTE_ADCBUFENABLE	equ 0x00000040	    ;; Enable ADC buffer interrupts                 ;;
INTE_EFXBUFENABLE	equ 0x00000020	    ;; Enable Effects buffer interrupts             ;;
INTE_GPSPDIFENABLE	equ 0x00000010	    ;; Enable GPSPDIF status interrupts             ;;
INTE_CDSPDIFENABLE	equ 0x00000008	    ;; Enable CDSPDIF status interrupts             ;;
INTE_INTERVALTIMERENB	equ 0x00000004	    ;; Enable interval timer interrupts             ;;
INTE_MIDITXENABLE	equ 0x00000002	    ;; Enable MIDI transmit-buffer-empty interrupts ;;
INTE_MIDIRXENABLE	equ 0x00000001	    ;; Enable MIDI receive-buffer-empty interrupts  ;;

WC			equ 0x10	    ;; Wall Clock register                          ;;
WC_SAMPLECOUNTER_MASK	equ 0x03FFFFC0	    ;; Sample periods elapsed since reset           ;;
WC_SAMPLECOUNTER	equ 0x14060010
WC_CURRENTCHANNEL	equ 0x0000003F	    ;; Channel [0..63] currently being serviced     ;;
						;; NOTE: Each channel takes 1/64th of a sample  ;;
						;; period to be serviced.                       ;;

HCFG			equ 0x14	    ; Hardware config register                     ;
						; NOTE: There is no reason to use the legacy   ;
						; SoundBlaster emulation stuff described below ;
						; under Linux, and all kinds of weird hardware ;
						; behavior can result if you try.  Don't.      ;
HCFG_LEGACYFUNC_MASK	equ 0xe0000000	    ; Legacy function number                       ;
HCFG_LEGACYFUNC_MPU	equ 0x00000000	    ; Legacy MPU                                   ;
HCFG_LEGACYFUNC_SB	equ 0x40000000	    ; Legacy SB                                    ;
HCFG_LEGACYFUNC_AD	equ 0x60000000	    ; Legacy AD                                    ;
HCFG_LEGACYFUNC_MPIC	equ 0x80000000	    ; Legacy MPIC                                  ;
HCFG_LEGACYFUNC_MDMA	equ 0xa0000000	    ; Legacy MDMA                                  ;
HCFG_LEGACYFUNC_SPCI	equ 0xc0000000	    ; Legacy SPCI                                  ;
HCFG_LEGACYFUNC_SDMA	equ 0xe0000000	    ; Legacy SDMA                                  ;
HCFG_IOCAPTUREADDR	equ 0x1f000000	    ; The 4 LSBs of the captured I/O address.      ;
HCFG_LEGACYWRITE	equ 0x00800000	    ; 1 = write, 0 = read                          ;
HCFG_LEGACYWORD 	equ 0x00400000	    ; 1 = word, 0 = byte                           ;
HCFG_LEGACYINT		equ 0x00200000	    ; 1 = legacy event captured. Write 1 to clear. ;
						; NOTE: The rest of the bits in this register  ;
						; _are_ relevant under Linux.                  ;
HCFG_PUSH_BUTTON_ENABLE equ 0x00100000	    ; Enables Volume Inc/Dec and Mute functions    ;
HCFG_BAUD_RATE		equ 0x00080000	    ; 0 = 48kHz, 1 = 44.1kHz                       ;
HCFG_EXPANDED_MEM	equ 0x00040000	    ; 1 = any 16M of 4G addr, 0 = 32M of 2G addr   ;
HCFG_CODECFORMAT_MASK	equ 0x00030000	    ; CODEC format                                 ;

; Specific to Alice2, CA0102 ;
HCFG_CODECFORMAT_AC97_1 equ 0x00000000	    ; AC97 CODEC format -- Ver 1.03                ;
HCFG_CODECFORMAT_AC97_2 equ 0x00010000	    ; AC97 CODEC format -- Ver 2.1                 ;
HCFG_AUTOMUTE_ASYNC	equ 0x00008000	    ; When set, the async sample rate convertors   ;
						; will automatically mute their output when    ;
						; they are not rate-locked to the external     ;
						; async audio source                           ;
HCFG_AUTOMUTE_SPDIF	equ 0x00004000	    ; When set, the async sample rate convertors   ;
						; will automatically mute their output when    ;
						; the SPDIF V-bit indicates invalid audio      ;
HCFG_EMU32_SLAVE	equ 0x00002000	    ; 0 = Master, 1 = Slave. Slave for EMU1010     ;
HCFG_SLOW_RAMP		equ 0x00001000	    ; Increases Send Smoothing time constant       ;
; 0x00000800 not used on Alice2 ;
HCFG_PHASE_TRACK_MASK	equ 0x00000700	    ; When set, forces corresponding input to      ;
						; phase track the previous input.              ;
						; I2S0 can phase track the last S/PDIF input   ;
HCFG_I2S_ASRC_ENABLE	equ 0x00000070	    ; When set, enables asynchronous sample rate   ;
						; conversion for the corresponding             ;
						; I2S format input                             ;
; Rest of HCFG 0x0000000f same as below. LOCKSOUNDCACHE etc.  ;



; Older chips ;
HCFG_CODECFORMAT_AC97	equ 0x00000000	    ; AC97 CODEC format -- Primary Output          ;
HCFG_CODECFORMAT_I2S	equ 0x00010000	    ; I2S CODEC format -- Secondary (Rear) Output  ;
HCFG_GPINPUT0		equ 0x00004000	    ; External pin112                              ;
HCFG_GPINPUT1		equ 0x00002000	    ; External pin110                              ;
HCFG_GPOUTPUT_MASK	equ 0x00001c00	    ; External pins which may be controlled        ;
HCFG_GPOUT0		equ 0x00001000	    ; External pin? (spdif enable on 5.1)          ;
HCFG_GPOUT1		equ 0x00000800	    ; External pin? (IR)                           ;
HCFG_GPOUT2		equ 0x00000400	    ; External pin? (IR)                           ;
HCFG_JOYENABLE		equ 0x00000200	    ; Internal joystick enable                     ;
HCFG_PHASETRACKENABLE	equ 0x00000100	    ; Phase tracking enable                        ;
						; 1 = Force all 3 async digital inputs to use  ;
						; the same async sample rate tracker (ZVIDEO)  ;
HCFG_AC3ENABLE_MASK	equ 0x000000e0	    ; AC3 async input control - Not implemented    ;
HCFG_AC3ENABLE_ZVIDEO	equ 0x00000080	    ; Channels 0 and 1 replace ZVIDEO              ;
HCFG_AC3ENABLE_CDSPDIF	equ 0x00000040	    ; Channels 0 and 1 replace CDSPDIF             ;
HCFG_AC3ENABLE_GPSPDIF	equ 0x00000020	    ; Channels 0 and 1 replace GPSPDIF             ;
HCFG_AUTOMUTE		equ 0x00000010	    ; When set, the async sample rate convertors   ;
						; will automatically mute their output when    ;
						; they are not rate-locked to the external     ;
						; async audio source                           ;
HCFG_LOCKSOUNDCACHE	equ 0x00000008	    ; 1 = Cancel bustmaster accesses to soundcache ;
						; NOTE: This should generally never be used.   ;
HCFG_LOCKTANKCACHE_MASK equ 0x00000004	    ; 1 = Cancel bustmaster accesses to tankcache  ;
						; NOTE: This should generally never be used.   ;
HCFG_LOCKTANKCACHE	equ 0x01020014
HCFG_MUTEBUTTONENABLE	equ 0x00000002	    ; 1 = Master mute button sets AUDIOENABLE = 0. ;
						; NOTE: This is a 'cheap' way to implement a   ;
						; master mute function on the mute button, and ;
						; in general should not be used unless a more  ;
						; sophisticated master mute function has not   ;
						; been written.                                ;
HCFG_AUDIOENABLE	equ 0x00000001	    ; 0 = CODECs transmit zero-valued samples      ;
						; Should be set to 1 when the EMU10K1 is       ;
						; completely initialized.                      ;

;;For Audigy, MPU port move to 0x70-0x74 ptr register

MUDATA			equ 0x18	    ;; MPU401 data register (8 bits)                ;;

MUCMD			equ 0x19	    ;; MPU401 command register (8 bits)             ;;
MUCMD_RESET		equ 0xFF	    ;; RESET command                                ;;
MUCMD_ENTERUARTMODE	equ 0x3F	    ;; Enter_UART_mode command                      ;;
						;; NOTE: All other commands are ignored         ;;

MUSTAT			equ MUCMD	    ;; MPU401 status register (8 bits)              ;;
MUSTAT_IRDYN		equ 0x80	    ;; 0 = MIDI data or command ACK                 ;;
MUSTAT_ORDYN		equ 0x40	    ;; 0 = MUDATA can accept a command or data      ;;

A_IOCFG 		equ 0x18	    ;; GPIO on Audigy card (16bits)                 ;;
A_GPINPUT_MASK		equ 0xFF00
A_GPOUTPUT_MASK 	equ 0x00FF

;; Audigy output/GPIO stuff taken from the kX drivers
A_IOCFG_GPOUT0		equ 0x0044	    ;; analog/digital                               ;;
A_IOCFG_DISABLE_ANALOG	equ 0x0040	    ;; = 'enable' for Audigy2 (chiprev=4)           ;;
A_IOCFG_ENABLE_DIGITAL	equ 0x0004
A_IOCFG_UNKNOWN_20	equ 0x0020
A_IOCFG_DISABLE_AC97_FRONT	equ 0x0080  ;; turn off ac97 front -> front (10k2.1)        ;;
A_IOCFG_GPOUT1		equ 0x0002	    ;; IR? drive's internal bypass (?)              ;;
A_IOCFG_GPOUT2		equ 0x0001	    ;; IR ;;
A_IOCFG_MULTIPURPOSE_JACK	equ 0x2000  ;; center+lfe+rear_center (a2/a2ex)             ;;
						;; + digital for generic 10k2                   ;;
A_IOCFG_DIGITAL_JACK	equ 0x1000	    ;; digital for a2 platinum                      ;;
A_IOCFG_FRONT_JACK	equ 0x4000
A_IOCFG_REAR_JACK	equ 0x8000
A_IOCFG_PHONES_JACK	equ 0x0100	    ;; LiveDrive                                    ;;

;; outputs:
;;      for audigy2 platinum:   0xA00
;;      for a2 platinum ex:     0x1C00
;;      for a1 platinum:        0x0

TIMER			equ 0x1A	    ;; Timer terminal count register                ;;
						;; NOTE: After the rate is changed, a maximum   ;;
						;; of 1024 sample periods should be allowed     ;;
						;; before the new rate is guaranteed accurate.  ;;
TIMER_RATE_MASK 	equ 0x000003FF	    ;; Timer interrupt rate in sample periods       ;;
						;; 0 == 1024 periods, [1..4] are not useful     ;;
TIMER_RATE		equ 0x0a00001A

AC97DATA		equ 0x1C	    ;; AC97 register set data register (16 bit)     ;;

AC97ADDRESS		equ 0x1E	    ;; AC97 register set address register (8 bit)   ;;
AC97ADDRESS_READY	equ 0x80	    ;; Read-only bit, reflects CODEC READY signal   ;;
AC97ADDRESS_ADDRESS	equ 0x7F	    ;; Address of indexed AC97 register             ;;

;; Available on the Audigy 2 and Audigy 4 only. This is the P16V chip. ;;
PTR2			equ 0x20	    ;; Indexed register set pointer register        ;;
DATA2			equ 0x24	    ;; Indexed register set data register           ;;
IPR2			equ 0x28	    ;; P16V interrupt pending register              ;;
IPR2_PLAYBACK_CH_0_LOOP      equ 0x00001000 ;; Playback Channel 0 loop                               ;;
IPR2_PLAYBACK_CH_0_HALF_LOOP equ 0x00000100 ;; Playback Channel 0 half loop                          ;;
IPR2_CAPTURE_CH_0_LOOP	     equ 0x00100000 ;; Capture Channel 0 loop                               ;;
IPR2_CAPTURE_CH_0_HALF_LOOP  equ 0x00010000 ;; Capture Channel 0 half loop                          ;;
						;; 0x00000100 Playback. Only in once per period.
						;; 0x00110000 Capture. Int on half buffer.
						 ;;
INTE2			equ 0x2C	    ;; P16V Interrupt enable register.      ;;
INTE2_PLAYBACK_CH_0_LOOP      equ 0x00001000 ;; Playback Channel 0 loop                               ;;
INTE2_PLAYBACK_CH_0_HALF_LOOP equ 0x00000100 ;; Playback Channel 0 half loop                          ;;
INTE2_PLAYBACK_CH_1_LOOP      equ 0x00002000 ;; Playback Channel 1 loop                               ;;
INTE2_PLAYBACK_CH_1_HALF_LOOP equ 0x00000200 ;; Playback Channel 1 half loop                          ;;
INTE2_PLAYBACK_CH_2_LOOP      equ 0x00004000 ;; Playback Channel 2 loop                               ;;
INTE2_PLAYBACK_CH_2_HALF_LOOP equ 0x00000400 ;; Playback Channel 2 half loop                          ;;
INTE2_PLAYBACK_CH_3_LOOP      equ 0x00008000 ;; Playback Channel 3 loop                               ;;
INTE2_PLAYBACK_CH_3_HALF_LOOP equ 0x00000800 ;; Playback Channel 3 half loop                          ;;
INTE2_CAPTURE_CH_0_LOOP       equ 0x00100000 ;; Capture Channel 0 loop                               ;;
INTE2_CAPTURE_CH_0_HALF_LOOP  equ 0x00010000 ;; Caputre Channel 0 half loop                          ;;
HCFG2			equ 0x34	    ;; Defaults: 0, win2000 sets it to 00004201 ;;
						;; 0x00000000 2-channel output. ;;
						;; 0x00000200 8-channel output. ;;
						;; 0x00000004 pauses stream/irq fail. ;;
						;; Rest of bits no nothing to sound output ;;
						;; bit 0: Enable P16V audio.
						;; bit 1: Lock P16V record memory cache.
						;; bit 2: Lock P16V playback memory cache.
						;; bit 3: Dummy record insert zero samples.
						;; bit 8: Record 8-channel in phase.
						;; bit 9: Playback 8-channel in phase.
						;; bit 11-12: Playback mixer attenuation: 0=0dB, 1=-6dB, 2=-12dB, 3=Mute.
						;; bit 13: Playback mixer enable.
						;; bit 14: Route SRC48 mixer output to fx engine.
						;; bit 15: Enable IEEE 1394 chip.
						 ;;
IPR3			equ 0x38	    ;; Cdif interrupt pending register              ;;
INTE3			equ 0x3C	    ;; Cdif interrupt enable register.      ;;

;;**********************************************************************************************;;
;; PCI function 1 registers, address = <val> + PCIBASE1                                         ;;
;;**********************************************************************************************;;

JOYSTICK1		equ 0x00	    ;; Analog joystick port register                ;;
JOYSTICK2		equ 0x01	    ;; Analog joystick port register                ;;
JOYSTICK3		equ 0x02	    ;; Analog joystick port register                ;;
JOYSTICK4		equ 0x03	    ;; Analog joystick port register                ;;
JOYSTICK5		equ 0x04	    ;; Analog joystick port register                ;;
JOYSTICK6		equ 0x05	    ;; Analog joystick port register                ;;
JOYSTICK7		equ 0x06	    ;; Analog joystick port register                ;;
JOYSTICK8		equ 0x07	    ;; Analog joystick port register                ;;

;; When writing, any write causes JOYSTICK_COMPARATOR output enable to be pulsed on write.      ;;
;; When reading, use these bitfields: ;;
JOYSTICK_BUTTONS	equ 0x0F	    ;; Joystick button data                         ;;
JOYSTICK_COMPARATOR	equ 0xF0	    ;; Joystick comparator data                     ;;


;;******************************************************************************************************;;
;; Emu10k1 pointer-offset register set, accessed through the PTR and DATA registers                     ;;
;;******************************************************************************************************;;

CPF			equ 0x00	    ;; Current pitch and fraction register                  ;;
CPF_CURRENTPITCH_MASK	equ 0xFFFF0000	    ;; Current pitch (linear, equ 0x4000 == unity pitch shift)  ;;
CPF_CURRENTPITCH	equ 0x10100000
CPF_STEREO_MASK 	equ 0x00008000	    ;; 1 = Even channel interleave, odd channel locked      ;;
CPF_STOP_MASK		equ 0x00004000	    ;; 1 = Current pitch forced to 0                        ;;
CPF_FRACADDRESS_MASK	equ 0x00003FFF	    ;; Linear fractional address of the current channel     ;;

PTRX			equ 0x01	    ;; Pitch target and send A/B amounts register           ;;
PTRX_PITCHTARGET_MASK	equ 0xFFFF0000	    ;; Pitch target of specified channel                    ;;
PTRX_PITCHTARGET	equ 0x10100001
PTRX_FXSENDAMOUNT_A_MASK equ 0x0000FF00     ;; Linear level of channel output sent to FX send bus A ;;
PTRX_FXSENDAMOUNT_A	equ 0x08080001
PTRX_FXSENDAMOUNT_B_MASK equ 0x000000FF     ;; Linear level of channel output sent to FX send bus B ;;
PTRX_FXSENDAMOUNT_B	equ 0x08000001

CVCF			equ 0x02	    ;; Current volume and filter cutoff register            ;;
CVCF_CURRENTVOL_MASK	equ 0xFFFF0000	    ;; Current linear volume of specified channel           ;;
CVCF_CURRENTVOL 	equ 0x10100002
CVCF_CURRENTFILTER_MASK equ 0x0000FFFF	    ;; Current filter cutoff frequency of specified channel ;;
CVCF_CURRENTFILTER	equ 0x10000002

VTFT			equ 0x03	    ;; Volume target and filter cutoff target register      ;;
VTFT_VOLUMETARGET_MASK	equ 0xFFFF0000	    ;; Volume target of specified channel                   ;;
VTFT_VOLUMETARGET	equ 0x10100003
VTFT_FILTERTARGET_MASK	equ 0x0000FFFF	    ;; Filter cutoff target of specified channel            ;;
VTFT_FILTERTARGET	equ 0x10000003

Z1			equ 0x05	    ;; Filter delay memory 1 register                       ;;

Z2			equ 0x04	    ;; Filter delay memory 2 register                       ;;

PSST			equ 0x06	    ;; Send C amount and loop start address register        ;;
PSST_FXSENDAMOUNT_C_MASK equ 0xFF000000     ;; Linear level of channel output sent to FX send bus C ;;

PSST_FXSENDAMOUNT_C	equ 0x08180006

PSST_LOOPSTARTADDR_MASK equ 0x00FFFFFF	    ;; Loop start address of the specified channel          ;;
PSST_LOOPSTARTADDR	equ 0x18000006

DSL			equ 0x07	    ;; Send D amount and loop start address register        ;;
DSL_FXSENDAMOUNT_D_MASK equ 0xFF000000	    ;; Linear level of channel output sent to FX send bus D ;;

DSL_FXSENDAMOUNT_D	equ 0x08180007

DSL_LOOPENDADDR_MASK	equ 0x00FFFFFF	    ;; Loop end address of the specified channel            ;;
DSL_LOOPENDADDR 	equ 0x18000007

CCCA			equ 0x08	    ;; Filter Q, interp. ROM, byte size, cur. addr register ;;
CCCA_RESONANCE		equ 0xF0000000	    ;; Lowpass filter resonance (Q) height                  ;;
CCCA_INTERPROMMASK	equ 0x0E000000	    ;; Selects passband of interpolation ROM                ;;
						;; 1 == full band, 7 == lowpass                         ;;
						;; ROM 0 is used when pitch shifting downward or less   ;;
						;; then 3 semitones upward.  Increasingly higher ROM    ;;
						;; numbers are used, typically in steps of 3 semitones, ;;
						;; as upward pitch shifting is performed.               ;;
CCCA_INTERPROM_0	equ 0x00000000	    ;; Select interpolation ROM 0                           ;;
CCCA_INTERPROM_1	equ 0x02000000	    ;; Select interpolation ROM 1                           ;;
CCCA_INTERPROM_2	equ 0x04000000	    ;; Select interpolation ROM 2                           ;;
CCCA_INTERPROM_3	equ 0x06000000	    ;; Select interpolation ROM 3                           ;;
CCCA_INTERPROM_4	equ 0x08000000	    ;; Select interpolation ROM 4                           ;;
CCCA_INTERPROM_5	equ 0x0A000000	    ;; Select interpolation ROM 5                           ;;
CCCA_INTERPROM_6	equ 0x0C000000	    ;; Select interpolation ROM 6                           ;;
CCCA_INTERPROM_7	equ 0x0E000000	    ;; Select interpolation ROM 7                           ;;
CCCA_8BITSELECT 	equ 0x01000000	    ;; 1 = Sound memory for this channel uses 8-bit samples ;;
CCCA_CURRADDR_MASK	equ 0x00FFFFFF	    ;; Current address of the selected channel              ;;
CCCA_CURRADDR		equ 0x18000008

CCR			equ 0x09	    ;; Cache control register                               ;;
CCR_CACHEINVALIDSIZE	equ 0x07190009
CCR_CACHEINVALIDSIZE_MASK	equ 0xFE000000	    ;; Number of invalid samples cache for this channel     ;;
CCR_CACHELOOPFLAG	equ 0x01000000	    ;; 1 = Cache has a loop service pending                 ;;
CCR_INTERLEAVEDSAMPLES	equ 0x00800000	    ;; 1 = A cache service will fetch interleaved samples   ;;
CCR_WORDSIZEDSAMPLES	equ 0x00400000	    ;; 1 = A cache service will fetch word sized samples    ;;
CCR_READADDRESS 	equ 0x06100009
CCR_READADDRESS_MASK	equ 0x003F0000	    ;; Location of cache just beyond current cache service  ;;
CCR_LOOPINVALSIZE	equ 0x0000FE00	    ;; Number of invalid samples in cache prior to loop     ;;
						;; NOTE: This is valid only if CACHELOOPFLAG is set     ;;
CCR_LOOPFLAG		equ 0x00000100	    ;; Set for a single sample period when a loop occurs    ;;
CCR_CACHELOOPADDRHI	equ 0x000000FF	    ;; DSL_LOOPSTARTADDR's hi byte if CACHELOOPFLAG is set  ;;

CLP			equ 0x0A	    ;; Cache loop register (valid if CCR_CACHELOOPFLAG = 1) ;;
						;; NOTE: This register is normally not used             ;;
CLP_CACHELOOPADDR	equ 0x0000FFFF	    ;; Cache loop address (DSL_LOOPSTARTADDR [0..15])       ;;

FXRT			equ 0x0B	    ;; Effects send routing register                        ;;
						;; NOTE: It is illegal to assign the same routing to    ;;
						;; two effects sends.                                   ;;
FXRT_CHANNELA		equ 0x000F0000	    ;; Effects send bus number for channel's effects send A ;;
FXRT_CHANNELB		equ 0x00F00000	    ;; Effects send bus number for channel's effects send B ;;
FXRT_CHANNELC		equ 0x0F000000	    ;; Effects send bus number for channel's effects send C ;;
FXRT_CHANNELD		equ 0xF0000000	    ;; Effects send bus number for channel's effects send D ;;

MAPA			equ 0x0C	    ;; Cache map A                                          ;;

MAPB			equ 0x0D	    ;; Cache map B                                          ;;

MAP_PTE_MASK		equ 0xFFFFE000	    ;; The 19 MSBs of the PTE indexed by the PTI            ;;
MAP_PTI_MASK		equ 0x00001FFF	    ;; The 13 bit index to one of the 8192 PTE dwords       ;;

ENVVOL			equ 0x10	    ;; Volume envelope register                             ;;
ENVVOL_MASK		equ 0x0000FFFF	    ;; Current value of volume envelope state variable      ;;
						;; equ 0x8000-n == 666*n usec delay                         ;;

ATKHLDV 		equ 0x11	    ;; Volume envelope hold and attack register             ;;
ATKHLDV_PHASE0		equ 0x00008000	    ;; 0 = Begin attack phase                               ;;
ATKHLDV_HOLDTIME_MASK	equ 0x00007F00	    ;; Envelope hold time (127-n == n*88.2msec)             ;;
ATKHLDV_ATTACKTIME_MASK equ 0x0000007F	    ;; Envelope attack time, log encoded                    ;;
						;; 0 = infinite, 1 = 10.9msec, ... equ 0x7f = 5.5msec       ;;

DCYSUSV 		equ 0x12	    ;; Volume envelope sustain and decay register           ;;
DCYSUSV_PHASE1_MASK	equ 0x00008000	    ;; 0 = Begin attack phase, 1 = begin release phase      ;;
DCYSUSV_SUSTAINLEVEL_MASK equ 0x00007F00    ;; 127 = full, 0 = off, 0.75dB increments               ;;
DCYSUSV_CHANNELENABLE_MASK equ 0x00000080   ;; 1 = Inhibit envelope engine from writing values in   ;;
						;; this channel and from writing to pitch, filter and   ;;
						;; volume targets.                                      ;;
DCYSUSV_DECAYTIME_MASK	equ 0x0000007F	    ;; Volume envelope decay time, log encoded              ;;
						;; 0 = 43.7msec, 1 = 21.8msec, equ 0x7f = 22msec            ;;

LFOVAL1 		equ 0x13	    ;; Modulation LFO value                                 ;;
LFOVAL_MASK		equ 0x0000FFFF	    ;; Current value of modulation LFO state variable       ;;
						;; equ 0x8000-n == 666*n usec delay                         ;;

ENVVAL			equ 0x14	    ;; Modulation envelope register                         ;;
ENVVAL_MASK		equ 0x0000FFFF	    ;; Current value of modulation envelope state variable  ;;
						;; equ 0x8000-n == 666*n usec delay                         ;;

ATKHLDM 		equ 0x15	    ;; Modulation envelope hold and attack register         ;;
ATKHLDM_PHASE0		equ 0x00008000	    ;; 0 = Begin attack phase                               ;;
ATKHLDM_HOLDTIME	equ 0x00007F00	    ;; Envelope hold time (127-n == n*42msec)               ;;
ATKHLDM_ATTACKTIME	equ 0x0000007F	    ;; Envelope attack time, log encoded                    ;;
						;; 0 = infinite, 1 = 11msec, ... equ 0x7f = 5.5msec         ;;

DCYSUSM 		equ 0x16	    ;; Modulation envelope decay and sustain register       ;;
DCYSUSM_PHASE1_MASK	equ 0x00008000	    ;; 0 = Begin attack phase, 1 = begin release phase      ;;
DCYSUSM_SUSTAINLEVEL_MASK equ 0x00007F00    ;; 127 = full, 0 = off, 0.75dB increments               ;;
DCYSUSM_DECAYTIME_MASK	equ 0x0000007F	    ;; Envelope decay time, log encoded                     ;;
						;; 0 = 43.7msec, 1 = 21.8msec, equ 0x7f = 22msec            ;;

LFOVAL2 		equ 0x17	    ;; Vibrato LFO register                                 ;;
LFOVAL2_MASK		equ 0x0000FFFF	    ;; Current value of vibrato LFO state variable          ;;
						;; equ 0x8000-n == 666*n usec delay                         ;;

IP			equ 0x18	    ;; Initial pitch register                               ;;
IP_MASK 		equ 0x0000FFFF	    ;; Exponential initial pitch shift                      ;;
						;; 4 bits of octave, 12 bits of fractional octave       ;;
IP_UNITY		equ 0x0000E000	    ;; Unity pitch shift                                    ;;

IFATN			equ 0x19	    ;; Initial filter cutoff and attenuation register       ;;
IFATN_FILTERCUTOFF_MASK equ 0x0000FF00	    ;; Initial filter cutoff frequency in exponential units ;;
						;; 6 most significant bits are semitones                ;;
						;; 2 least significant bits are fractions               ;;
IFATN_FILTERCUTOFF	equ 0x08080019
IFATN_ATTENUATION_MASK	equ 0x000000FF	    ;; Initial attenuation in 0.375dB steps                 ;;
IFATN_ATTENUATION	equ 0x08000019


PEFE			equ 0x1A	    ;; Pitch envelope and filter envelope amount register   ;;
PEFE_PITCHAMOUNT_MASK	equ 0x0000FF00	    ;; Pitch envlope amount                                 ;;
						;; Signed 2's complement, +/- one octave peak extremes  ;;
PEFE_PITCHAMOUNT	equ 0x0808001A
PEFE_FILTERAMOUNT_MASK	equ 0x000000FF	    ;; Filter envlope amount                                ;;
						;; Signed 2's complement, +/- six octaves peak extremes ;;
PEFE_FILTERAMOUNT	equ 0x0800001A
FMMOD			equ 0x1B	    ;; Vibrato/filter modulation from LFO register          ;;
FMMOD_MODVIBRATO	equ 0x0000FF00	    ;; Vibrato LFO modulation depth                         ;;
						;; Signed 2's complement, +/- one octave extremes       ;;
FMMOD_MOFILTER		equ 0x000000FF	    ;; Filter LFO modulation depth                          ;;
						;; Signed 2's complement, +/- three octave extremes     ;;


TREMFRQ 		equ 0x1C	    ;; Tremolo amount and modulation LFO frequency register ;;
TREMFRQ_DEPTH		equ 0x0000FF00	    ;; Tremolo depth                                        ;;
						;; Signed 2's complement, with +/- 12dB extremes        ;;

TREMFRQ_FREQUENCY	equ 0x000000FF	    ;; Tremolo LFO frequency                                ;;
						;; ??Hz steps, maximum of ?? Hz.                        ;;
FM2FRQ2 		equ 0x1D	    ;; Vibrato amount and vibrato LFO frequency register    ;;
FM2FRQ2_DEPTH		equ 0x0000FF00	    ;; Vibrato LFO vibrato depth                            ;;
						;; Signed 2's complement, +/- one octave extremes       ;;
FM2FRQ2_FREQUENCY	equ 0x000000FF	    ;; Vibrato LFO frequency                                ;;
						;; 0.039Hz steps, maximum of 9.85 Hz.                   ;;

TEMPENV 		equ 0x1E	    ;; Tempory envelope register                            ;;
TEMPENV_MASK		equ 0x0000FFFF	    ;; 16-bit value                                         ;;
						;; NOTE: All channels contain internal variables; do    ;;
						;; not write to these locations.                        ;;

;; 1f something ;;

CD0			equ 0x20	    ;; Cache data 0 register                                ;;
CD1			equ 0x21	    ;; Cache data 1 register                                ;;
CD2			equ 0x22	    ;; Cache data 2 register                                ;;
CD3			equ 0x23	    ;; Cache data 3 register                                ;;
CD4			equ 0x24	    ;; Cache data 4 register                                ;;
CD5			equ 0x25	    ;; Cache data 5 register                                ;;
CD6			equ 0x26	    ;; Cache data 6 register                                ;;
CD7			equ 0x27	    ;; Cache data 7 register                                ;;
CD8			equ 0x28	    ;; Cache data 8 register                                ;;
CD9			equ 0x29	    ;; Cache data 9 register                                ;;
CDA			equ 0x2A	    ;; Cache data A register                                ;;
CDB			equ 0x2B	    ;; Cache data B register                                ;;
CDC			equ 0x2C	    ;; Cache data C register                                ;;
CDD			equ 0x2D	    ;; Cache data D register                                ;;
CDE			equ 0x2E	    ;; Cache data E register                                ;;
CDF			equ 0x2F	    ;; Cache data F register                                ;;

;; equ 0x30-3f seem to be the same as equ 0x20-2f ;;

PTB			equ 0x40	    ;; Page table base register                             ;;
PTB_MASK		equ 0xFFFFF000	    ;; Physical address of the page table in host memory    ;;

TCB			equ 0x41	    ;; Tank cache base register                             ;;
TCB_MASK		equ 0xFFFFF000	    ;; Physical address of the bottom of host based TRAM    ;;

ADCCR			equ 0x42	    ;; ADC sample rate/stereo control register              ;;
ADCCR_RCHANENABLE	equ 0x00000010	    ;; Enables right channel for writing to the host        ;;
ADCCR_LCHANENABLE	equ 0x00000008	    ;; Enables left channel for writing to the host         ;;
						;; NOTE: To guarantee phase coherency, both channels    ;;
						;; must be disabled prior to enabling both channels.    ;;
A_ADCCR_RCHANENABLE	equ 0x00000020
A_ADCCR_LCHANENABLE	equ 0x00000010

A_ADCCR_SAMPLERATE_MASK equ 0x0000000F	    ;; Audigy sample rate convertor output rate             ;;
ADCCR_SAMPLERATE_MASK	equ 0x00000007	    ;; Sample rate convertor output rate                    ;;
ADCCR_SAMPLERATE_48	equ 0x00000000	    ;; 48kHz sample rate                                    ;;
ADCCR_SAMPLERATE_44	equ 0x00000001	    ;; 44.1kHz sample rate                                  ;;
ADCCR_SAMPLERATE_32	equ 0x00000002	    ;; 32kHz sample rate                                    ;;
ADCCR_SAMPLERATE_24	equ 0x00000003	    ;; 24kHz sample rate                                    ;;
ADCCR_SAMPLERATE_22	equ 0x00000004	    ;; 22.05kHz sample rate                                 ;;
ADCCR_SAMPLERATE_16	equ 0x00000005	    ;; 16kHz sample rate                                    ;;
ADCCR_SAMPLERATE_11	equ 0x00000006	    ;; 11.025kHz sample rate                                ;;
ADCCR_SAMPLERATE_8	equ 0x00000007	    ;; 8kHz sample rate                                     ;;
A_ADCCR_SAMPLERATE_12	equ 0x00000006	    ;; 12kHz sample rate                                    ;;
A_ADCCR_SAMPLERATE_11	equ 0x00000007	    ;; 11.025kHz sample rate                                ;;
A_ADCCR_SAMPLERATE_8	equ 0x00000008	    ;; 8kHz sample rate                                     ;;

FXWC			equ 0x43	    ;; FX output write channels register                    ;;
						;; When set, each bit enables the writing of the        ;;
						;; corresponding FX output channel (internal registers  ;;
						;; equ 0x20-equ 0x3f) to host memory.  This mode of recording   ;;
						;; is 16bit, 48KHz only. All 32 channels can be enabled ;;
						;; simultaneously.                                      ;;

FXWC_DEFAULTROUTE_C	equ (1<<0)	    ;; left emu out? ;;
FXWC_DEFAULTROUTE_B	equ (1<<1)	    ;; right emu out? ;;
FXWC_DEFAULTROUTE_A	equ (1<<12)
FXWC_DEFAULTROUTE_D	equ (1<<13)
FXWC_ADCLEFT		equ (1<<18)
FXWC_CDROMSPDIFLEFT	equ (1<<18)
FXWC_ADCRIGHT		equ (1<<19)
FXWC_CDROMSPDIFRIGHT	equ (1<<19)
FXWC_MIC		equ (1<<20)
FXWC_ZOOMLEFT		equ (1<<20)
FXWC_ZOOMRIGHT		equ (1<<21)
FXWC_SPDIFLEFT		equ (1<<22)	    ;; equ 0x00400000 ;;
FXWC_SPDIFRIGHT 	equ (1<<23)	    ;; equ 0x00800000 ;;

TCBS			equ 0x44	    ;; Tank cache buffer size register                      ;;
TCBS_MASK		equ 0x00000007	    ;; Tank cache buffer size field                         ;;
TCBS_BUFFSIZE_16K	equ 0x00000000
TCBS_BUFFSIZE_32K	equ 0x00000001
TCBS_BUFFSIZE_64K	equ 0x00000002
TCBS_BUFFSIZE_128K	equ 0x00000003
TCBS_BUFFSIZE_256K	equ 0x00000004
TCBS_BUFFSIZE_512K	equ 0x00000005
TCBS_BUFFSIZE_1024K	equ 0x00000006
TCBS_BUFFSIZE_2048K	equ 0x00000007

MICBA			equ 0x45	    ;; AC97 microphone buffer address register              ;;
MICBA_MASK		equ 0xFFFFF000	    ;; 20 bit base address                                  ;;

ADCBA			equ 0x46	    ;; ADC buffer address register                          ;;
ADCBA_MASK		equ 0xFFFFF000	    ;; 20 bit base address                                  ;;

FXBA			equ 0x47	    ;; FX Buffer Address ;;
FXBA_MASK		equ 0xFFFFF000	    ;; 20 bit base address                                  ;;

;; equ 0x48 something - word access, defaults to 3f ;;

MICBS			equ 0x49	    ;; Microphone buffer size register                      ;;

ADCBS			equ 0x4A	    ;; ADC buffer size register                             ;;

FXBS			equ 0x4B	    ;; FX buffer size register                              ;;

;; register: equ 0x4C..4F: FFFF-FFFF current amounts, per-channel ;;

;; The following mask values define the size of the ADC, MIX and FX buffers in bytes ;;
ADCBS_BUFSIZE_NONE	equ 0x00000000
ADCBS_BUFSIZE_384	equ 0x00000001
ADCBS_BUFSIZE_448	equ 0x00000002
ADCBS_BUFSIZE_512	equ 0x00000003
ADCBS_BUFSIZE_640	equ 0x00000004
ADCBS_BUFSIZE_768	equ 0x00000005
ADCBS_BUFSIZE_896	equ 0x00000006
ADCBS_BUFSIZE_1024	equ 0x00000007
ADCBS_BUFSIZE_1280	equ 0x00000008
ADCBS_BUFSIZE_1536	equ 0x00000009
ADCBS_BUFSIZE_1792	equ 0x0000000A
ADCBS_BUFSIZE_2048	equ 0x0000000B
ADCBS_BUFSIZE_2560	equ 0x0000000C
ADCBS_BUFSIZE_3072	equ 0x0000000D
ADCBS_BUFSIZE_3584	equ 0x0000000E
ADCBS_BUFSIZE_4096	equ 0x0000000F
ADCBS_BUFSIZE_5120	equ 0x00000010
ADCBS_BUFSIZE_6144	equ 0x00000011
ADCBS_BUFSIZE_7168	equ 0x00000012
ADCBS_BUFSIZE_8192	equ 0x00000013
ADCBS_BUFSIZE_10240	equ 0x00000014
ADCBS_BUFSIZE_12288	equ 0x00000015
ADCBS_BUFSIZE_14366	equ 0x00000016
ADCBS_BUFSIZE_16384	equ 0x00000017
ADCBS_BUFSIZE_20480	equ 0x00000018
ADCBS_BUFSIZE_24576	equ 0x00000019
ADCBS_BUFSIZE_28672	equ 0x0000001A
ADCBS_BUFSIZE_32768	equ 0x0000001B
ADCBS_BUFSIZE_40960	equ 0x0000001C
ADCBS_BUFSIZE_49152	equ 0x0000001D
ADCBS_BUFSIZE_57344	equ 0x0000001E
ADCBS_BUFSIZE_65536	equ 0x0000001F


CDCS			equ 0x50	    ;; CD-ROM digital channel status register       ;;

GPSCS			equ 0x51	    ;; General Purpose SPDIF channel status register;;

DBG			equ 0x52	    ;; DO NOT PROGRAM THIS REGISTER!!! MAY DESTROY CHIP ;;

REG53			equ 0x53	    ;; DO NOT PROGRAM THIS REGISTER!!! MAY DESTROY CHIP ;;

A_DBG			 equ 0x53
A_DBG_SINGLE_STEP	 equ 0x00020000     ;; Set to zero to start dsp ;;
A_DBG_ZC		 equ 0x40000000     ;; zero tram counter ;;
A_DBG_STEP_ADDR 	 equ 0x000003FF
A_DBG_SATURATION_OCCURED equ 0x20000000
A_DBG_SATURATION_ADDR	 equ 0x0FFC0000

;; NOTE: equ 0x54,55,56: 64-bit
SPCS0			equ 0x54	    ;; SPDIF output Channel Status 0 register       ;;

SPCS1			equ 0x55	    ;; SPDIF output Channel Status 1 register       ;;

SPCS2			equ 0x56	    ;; SPDIF output Channel Status 2 register       ;;

SPCS_CLKACCYMASK	equ 0x30000000	    ;; Clock accuracy                               ;;
SPCS_CLKACCY_1000PPM	equ 0x00000000	    ;; 1000 parts per million                       ;;
SPCS_CLKACCY_50PPM	equ 0x10000000	    ;; 50 parts per million                         ;;
SPCS_CLKACCY_VARIABLE	equ 0x20000000	    ;; Variable accuracy                            ;;
SPCS_SAMPLERATEMASK	equ 0x0F000000	    ;; Sample rate                                  ;;
SPCS_SAMPLERATE_44	equ 0x00000000	    ;; 44.1kHz sample rate                          ;;
SPCS_SAMPLERATE_48	equ 0x02000000	    ;; 48kHz sample rate                            ;;
SPCS_SAMPLERATE_32	equ 0x03000000	    ;; 32kHz sample rate                            ;;
SPCS_CHANNELNUMMASK	equ 0x00F00000	    ;; Channel number                               ;;
SPCS_CHANNELNUM_UNSPEC	equ 0x00000000	    ;; Unspecified channel number                   ;;
SPCS_CHANNELNUM_LEFT	equ 0x00100000	    ;; Left channel                                 ;;
SPCS_CHANNELNUM_RIGHT	equ 0x00200000	    ;; Right channel                                ;;
SPCS_SOURCENUMMASK	equ 0x000F0000	    ;; Source number                                ;;
SPCS_SOURCENUM_UNSPEC	equ 0x00000000	    ;; Unspecified source number                    ;;
SPCS_GENERATIONSTATUS	equ 0x00008000	    ;; Originality flag (see IEC-958 spec)          ;;
SPCS_CATEGORYCODEMASK	equ 0x00007F00	    ;; Category code (see IEC-958 spec)             ;;
SPCS_MODEMASK		equ 0x000000C0	    ;; Mode (see IEC-958 spec)                      ;;
SPCS_EMPHASISMASK	equ 0x00000038	    ;; Emphasis                                     ;;
SPCS_EMPHASIS_NONE	equ 0x00000000	    ;; No emphasis                                  ;;
SPCS_EMPHASIS_50_15	equ 0x00000008	    ;; 50/15 usec 2 channel                         ;;
SPCS_COPYRIGHT		equ 0x00000004	    ;; Copyright asserted flag -- do not modify     ;;
SPCS_NOTAUDIODATA	equ 0x00000002	    ;; 0 = Digital audio, 1 = not audio             ;;
SPCS_PROFESSIONAL	equ 0x00000001	    ;; 0 = Consumer (IEC-958), 1 = pro (AES3-1992)  ;;

;; The 32-bit CLIx and SOLx registers all have one bit per channel control/status               ;;
CLIEL			equ 0x58	    ;; Channel loop interrupt enable low register   ;;

CLIEH			equ 0x59	    ;; Channel loop interrupt enable high register  ;;

CLIPL			equ 0x5A	    ;; Channel loop interrupt pending low register  ;;

CLIPH			equ 0x5B	    ;; Channel loop interrupt pending high register ;;

SOLEL			equ 0x5C	    ;; Stop on loop enable low register             ;;

SOLEH			equ 0x5D	    ;; Stop on loop enable high register            ;;

SPBYPASS		equ 0x5E	    ;; SPDIF BYPASS mode register                   ;;
SPBYPASS_SPDIF0_MASK	equ 0x00000003	    ;; SPDIF 0 bypass mode                          ;;
SPBYPASS_SPDIF1_MASK	equ 0x0000000C	    ;; SPDIF 1 bypass mode                          ;;
;; bypass mode: 0 - DSP; 1 - SPDIF A, 2 - SPDIF B, 3 - SPDIF C                                  ;;
SPBYPASS_FORMAT 	equ 0x00000f00	    ;; If 1, SPDIF XX uses 24 bit, if 0 - 20 bit    ;;

AC97SLOT		equ 0x5F	    ;; additional AC97 slots enable bits            ;;
AC97SLOT_REAR_RIGHT	equ 0x01	    ;; Rear left ;;
AC97SLOT_REAR_LEFT	equ 0x02	    ;; Rear right ;;
AC97SLOT_CNTR		equ 0x10	    ;; Center enable ;;
AC97SLOT_LFE		equ 0x20	    ;; LFE enable ;;

;; NOTE: equ 0x60,61,62: 64-bit
CDSRCS			equ 0x60	    ;; CD-ROM Sample Rate Converter status register ;;

GPSRCS			equ 0x61	    ;; General Purpose SPDIF sample rate cvt status ;;

ZVSRCS			equ 0x62	    ;; ZVideo sample rate converter status          ;;
						;; NOTE: This one has no SPDIFLOCKED field      ;;
						;; Assumes sample lock                          ;;

;; These three bitfields apply to CDSRCS, GPSRCS, and (except as noted) ZVSRCS.                 ;;
SRCS_SPDIFVALID 	equ 0x04000000	    ;; SPDIF stream valid                           ;;
SRCS_SPDIFLOCKED	equ 0x02000000	    ;; SPDIF stream locked                          ;;
SRCS_RATELOCKED 	equ 0x01000000	    ;; Sample rate locked                           ;;
SRCS_ESTSAMPLERATE	equ 0x0007FFFF	    ;; Do not modify this field.                    ;;

;; Note that these values can vary +/- by a small amount                                        ;;
SRCS_SPDIFRATE_44	equ 0x0003ACD9
SRCS_SPDIFRATE_48	equ 0x00040000
SRCS_SPDIFRATE_96	equ 0x00080000

MICIDX			equ 0x63	    ;; Microphone recording buffer index register   ;;
MICIDX_MASK		equ 0x0000FFFF	    ;; 16-bit value                                 ;;
MICIDX_IDX		equ 0x10000063

ADCIDX			equ 0x64	    ;; ADC recording buffer index register          ;;
ADCIDX_MASK		equ 0x0000FFFF	    ;; 16 bit index field                           ;;
ADCIDX_IDX		equ 0x10000064

A_ADCIDX		equ 0x63
A_ADCIDX_IDX		equ 0x10000063

A_MICIDX		equ 0x64
A_MICIDX_IDX		equ 0x10000064

FXIDX			equ 0x65	    ;; FX recording buffer index register           ;;
FXIDX_MASK		equ 0x0000FFFF	    ;; 16-bit value                                 ;;
FXIDX_IDX		equ 0x10000065

;; The 32-bit HLIx and HLIPx registers all have one bit per channel control/status                      ;;
HLIEL			equ 0x66	    ;; Channel half loop interrupt enable low register      ;;

HLIEH			equ 0x67	    ;; Channel half loop interrupt enable high register     ;;

HLIPL			equ 0x68	    ;; Channel half loop interrupt pending low register     ;;

HLIPH			equ 0x69	    ;; Channel half loop interrupt pending high register    ;;

;; equ 0x6a,6b,6c used for some recording
;; equ 0x6d unused
;; equ 0x6e,6f - tanktable base / offset

;; This is the MPU port on the card (via the game port)                                         ;;
A_MUDATA1		equ 0x70
A_MUCMD1		equ 0x71
A_MUSTAT1		equ A_MUCMD1

;; This is the MPU port on the Audigy Drive                                                     ;;
A_MUDATA2		equ 0x72
A_MUCMD2		equ 0x73
A_MUSTAT2		equ A_MUCMD2

;; The next two are the Audigy equivalent of FXWC                                               ;;
;; the Audigy can record any output (16bit, 48kHz, up to 64 channel simultaneously)             ;;
;; Each bit selects a channel for recording ;;
A_FXWC1 		equ 0x74	    ;; Selects equ 0x7f-equ 0x60 for FX recording           ;;
A_FXWC2 		equ 0x75	    ;; Selects equ 0x9f-equ 0x80 for FX recording           ;;

A_SPDIF_SAMPLERATE	equ 0x76	    ;; Set the sample rate of SPDIF output          ;;
A_SAMPLE_RATE		equ 0x76	    ;; Various sample rate settings. ;;
A_SAMPLE_RATE_NOT_USED	equ 0x0FFC111E	    ;; Bits that are not used and cannot be set.    ;;
A_SAMPLE_RATE_UNKNOWN	equ 0xF0030001	    ;; Bits that can be set, but have unknown use.  ;;
A_SPDIF_RATE_MASK	equ 0x000000E0	    ;; Any other values for rates, just use 48000   ;;
A_SPDIF_48000		equ 0x00000000
A_SPDIF_192000		equ 0x00000020
A_SPDIF_96000		equ 0x00000040
A_SPDIF_44100		equ 0x00000080

A_I2S_CAPTURE_RATE_MASK equ 0x00000E00	    ;; This sets the capture PCM rate, but it is    ;;
A_I2S_CAPTURE_48000	equ 0x00000000	    ;; unclear if this sets the ADC rate as well.   ;;
A_I2S_CAPTURE_192000	equ 0x00000200
A_I2S_CAPTURE_96000	equ 0x00000400
A_I2S_CAPTURE_44100	equ 0x00000800

A_PCM_RATE_MASK 	equ 0x0000E000	    ;; This sets the playback PCM rate on the P16V  ;;
A_PCM_48000		equ 0x00000000
A_PCM_192000		equ 0x00002000
A_PCM_96000		equ 0x00004000
A_PCM_44100		equ 0x00008000

;; equ 0x77,equ 0x78,equ 0x79 "something i2s-related" - default to equ 0x01080000 on my audigy 2 ZS --rlrevell  ;;
;; equ 0x7A, equ 0x7B - lookup tables ;;

A_FXRT2 		equ 0x7C
A_FXRT_CHANNELE 	equ 0x0000003F	    ;; Effects send bus number for channel's effects send E ;;
A_FXRT_CHANNELF 	equ 0x00003F00	    ;; Effects send bus number for channel's effects send F ;;
A_FXRT_CHANNELG 	equ 0x003F0000	    ;; Effects send bus number for channel's effects send G ;;
A_FXRT_CHANNELH 	equ 0x3F000000	    ;; Effects send bus number for channel's effects send H ;;

A_SENDAMOUNTS		equ 0x7D
A_FXSENDAMOUNT_E_MASK	equ 0xFF000000
A_FXSENDAMOUNT_F_MASK	equ 0x00FF0000
A_FXSENDAMOUNT_G_MASK	equ 0x0000FF00
A_FXSENDAMOUNT_H_MASK	equ 0x000000FF
;; equ 0x7C, equ 0x7E "high bit is used for filtering" ;;
 
;; The send amounts for this one are the same as used with the emu10k1 ;;
A_FXRT1 		equ 0x7E
A_FXRT_CHANNELA 	equ 0x0000003F
A_FXRT_CHANNELB 	equ 0x00003F00
A_FXRT_CHANNELC 	equ 0x003F0000
A_FXRT_CHANNELD 	equ 0x3F000000


;; Each FX general purpose register is 32 bits in length, all bits are used                     ;;
FXGPREGBASE		equ 0x100	    ;; FX general purpose registers base            ;;
A_FXGPREGBASE		equ 0x400	    ;; Audigy GPRs, equ 0x400 to equ 0x5FF                  ;;

A_TANKMEMCTLREGBASE	equ 0x100	    ;; Tank memory control registers base - only for Audigy ;;
A_TANKMEMCTLREG_MASK	equ 0x1F	    ;; only 5 bits used - only for Audigy ;;

;; Tank audio data is logarithmically compressed down to 16 bits before writing to TRAM and is  ;;
;; decompressed back to 20 bits on a read.  There are a total of 160 locations, the last 32     ;;
;; locations are for external TRAM.                                                             ;;
TANKMEMDATAREGBASE	equ 0x200	    ;; Tank memory data registers base              ;;
TANKMEMDATAREG_MASK	equ 0x000FFFFF	    ;; 20 bit tank audio data field                 ;;

;; Combined address field and memory opcode or flag field.  160 locations, last 32 are external ;;
TANKMEMADDRREGBASE	equ 0x300	    ;; Tank memory address registers base           ;;
TANKMEMADDRREG_ADDR_MASK equ 0x000FFFFF     ;; 20 bit tank address field                    ;;
TANKMEMADDRREG_CLEAR	equ 0x00800000	    ;; Clear tank memory                            ;;
TANKMEMADDRREG_ALIGN	equ 0x00400000	    ;; Align read or write relative to tank access  ;;
TANKMEMADDRREG_WRITE	equ 0x00200000	    ;; Write to tank memory                         ;;
TANKMEMADDRREG_READ	equ 0x00100000	    ;; Read from tank memory                        ;;

MICROCODEBASE		equ 0x400	    ;; Microcode data base address                  ;;

;; Each DSP microcode instruction is mapped into 2 doublewords                                  ;;
;; NOTE: When writing, always write the LO doubleword first.  Reads can be in either order.     ;;
LOWORD_OPX_MASK 	equ 0x000FFC00	    ;; Instruction operand X                        ;;
LOWORD_OPY_MASK 	equ 0x000003FF	    ;; Instruction operand Y                        ;;
HIWORD_OPCODE_MASK	equ 0x00F00000	    ;; Instruction opcode                           ;;
HIWORD_RESULT_MASK	equ 0x000FFC00	    ;; Instruction result                           ;;
HIWORD_OPA_MASK 	equ 0x000003FF	    ;; Instruction operand A                        ;;


;; Audigy Soundcard have a different instruction format ;;
A_MICROCODEBASE 	equ 0x600
A_LOWORD_OPY_MASK	equ 0x000007FF
A_LOWORD_OPX_MASK	equ 0x007FF000
A_HIWORD_OPCODE_MASK	equ 0x0F000000
A_HIWORD_RESULT_MASK	equ 0x007FF000
A_HIWORD_OPA_MASK	equ 0x000007FF
 

macro snd_emu10k1_compose_send_routing route
{
 ;((route[0] | (route[1] << 4) | (route[2] << 8) | (route[3] << 12)) << 16)
   push    ebx
   movzx   eax, byte [route+3]
   shl	   eax, 12
   movzx   ebx, byte [route+2]
   shl	   ebx, 8
   or	   eax, ebx
   movzx   ebx, byte [route+1]
   shl	   ebx, 4
   or	   eax, ebx
   or	   al, byte [route]
   shl	   eax, 16
   pop	   ebx
}

