
;******************************************************************************;
;* Audigy2Value Tina (P17V) pointer-offset register set,
;* accessed through the PTR20 and DATA24 registers  *;
;******************************************************************************;

; 00 - 07: Not used ;
P17V_PLAYBACK_FIFO_PTR	equ 0x08    ; Current playback fifo pointer
					 ; and number of sound samples in cache.
					 ;
; 09 - 12: Not used ;
P17V_CAPTURE_FIFO_PTR	equ 0x13    ; Current capture fifo pointer
					 ; and number of sound samples in cache.
					 ;
; 14 - 17: Not used ;
P17V_PB_CHN_SEL 	equ 0x18    ; P17v playback channel select ;
P17V_SE_SLOT_SEL_L	equ 0x19    ; Sound Engine slot select low ;
P17V_SE_SLOT_SEL_H	equ 0x1a    ; Sound Engine slot select high ;
; 1b - 1f: Not used ;
; 20 - 2f: Not used ;
; 30 - 3b: Not used ;
P17V_SPI		equ 0x3c    ; SPI interface register ;
P17V_I2C_ADDR		equ 0x3d    ; I2C Address ;
P17V_I2C_0		equ 0x3e    ; I2C Data ;
P17V_I2C_1		equ 0x3f    ; I2C Data ;
; I2C values ;
I2C_A_ADC_ADD_MASK	equ 0x000000fe	    ;The address is a 7 bit address ;
I2C_A_ADC_RW_MASK	equ 0x00000001	    ;bit mask for R/W ;
I2C_A_ADC_TRANS_MASK	equ 0x00000010	    ;Bit mask for I2c address DAC value  ;
I2C_A_ADC_ABORT_MASK	equ 0x00000020	    ;Bit mask for I2C transaction abort flag ;
I2C_A_ADC_LAST_MASK	equ 0x00000040	    ;Bit mask for Last word transaction ;
I2C_A_ADC_BYTE_MASK	equ 0x00000080	    ;Bit mask for Byte Mode ;

I2C_A_ADC_ADD		equ 0x00000034	    ;This is the Device address for ADC  ;
I2C_A_ADC_READ		equ 0x00000001	    ;To perform a read operation ;
I2C_A_ADC_START 	equ 0x00000100	    ;Start I2C transaction ;
I2C_A_ADC_ABORT 	equ 0x00000200	    ;I2C transaction abort ;
I2C_A_ADC_LAST		equ 0x00000400	    ;I2C last transaction ;
I2C_A_ADC_BYTE		equ 0x00000800	    ;I2C one byte mode ;

I2C_D_ADC_REG_MASK	equ 0xfe000000	    ;ADC address register ;
I2C_D_ADC_DAT_MASK	equ 0x01ff0000	    ;ADC data register ;

ADC_TIMEOUT		equ 0x00000007	    ;ADC Timeout Clock Disable ;
ADC_IFC_CTRL		equ 0x0000000b	    ;ADC Interface Control ;
ADC_MASTER		equ 0x0000000c	    ;ADC Master Mode Control ;
ADC_POWER		equ 0x0000000d	    ;ADC PowerDown Control ;
ADC_ATTEN_ADCL		equ 0x0000000e	    ;ADC Attenuation ADCL ;
ADC_ATTEN_ADCR		equ 0x0000000f	    ;ADC Attenuation ADCR ;
ADC_ALC_CTRL1		equ 0x00000010	    ;ADC ALC Control 1 ;
ADC_ALC_CTRL2		equ 0x00000011	    ;ADC ALC Control 2 ;
ADC_ALC_CTRL3		equ 0x00000012	    ;ADC ALC Control 3 ;
ADC_NOISE_CTRL		equ 0x00000013	    ;ADC Noise Gate Control ;
ADC_LIMIT_CTRL		equ 0x00000014	    ;ADC Limiter Control ;
ADC_MUX 		equ 0x00000015	    ;ADC Mux offset ;
if 0
; FIXME: Not tested yet. ;
ADC_GAIN_MASK		equ 0x000000ff	    ;Mask for ADC Gain
ADC_ZERODB		equ 0x000000cf	    ;Value to set ADC to 0dB
ADC_MUTE_MASK		equ 0x000000c0	    ;Mask for ADC mute
ADC_MUTE		equ 0x000000c0	    ;Value to mute ADC
ADC_OSR 		equ 0x00000008	    ;Mask for ADC oversample rate select
ADC_TIMEOUT_DISABLE	equ 0x00000008	    ;Value and mask to disable Timeout clock
ADC_HPF_DISABLE 	equ 0x00000100	    ;Value and mask to disable High pass filter
ADC_TRANWIN_MASK	equ 0x00000070	    ;Mask for Length of Transient Window
end if

ADC_MUX_MASK		equ 0x0000000f	    ;Mask for ADC Mux
ADC_MUX_0		equ 0x00000001	    ;Value to select Unknown at ADC Mux (Not used)
ADC_MUX_1		equ 0x00000002	    ;Value to select Unknown at ADC Mux (Not used)
ADC_MUX_2		equ 0x00000004	    ;Value to select Mic at ADC Mux
ADC_MUX_3		equ 0x00000008	    ;Value to select Line-In at ADC Mux

P17V_START_AUDIO	equ 0x40    ; Start Audio bit ;
; 41 - 47: Reserved ;
P17V_START_CAPTURE	equ 0x48    ; Start Capture bit ;
P17V_CAPTURE_FIFO_BASE	equ 0x49    ; Record FIFO base address ;
P17V_CAPTURE_FIFO_SIZE	equ 0x4a    ; Record FIFO buffer size ;
P17V_CAPTURE_FIFO_INDEX equ 0x4b    ; Record FIFO capture index ;
P17V_CAPTURE_VOL_H	equ 0x4c    ; P17v capture volume control ;
P17V_CAPTURE_VOL_L	equ 0x4d    ; P17v capture volume control ;
; 4e - 4f: Not used ;
; 50 - 5f: Not used ;
P17V_SRCSel		equ 0x60    ; SRC48 and SRCMulti sample rate select
					 ; and output select
					 ;
P17V_MIXER_AC97_10K1_VOL_L	equ 0x61    ; 10K to Mixer_AC97 input volume control ;
P17V_MIXER_AC97_10K1_VOL_H	equ 0x62    ; 10K to Mixer_AC97 input volume control ;
P17V_MIXER_AC97_P17V_VOL_L	equ 0x63    ; P17V to Mixer_AC97 input volume control ;
P17V_MIXER_AC97_P17V_VOL_H	equ 0x64    ; P17V to Mixer_AC97 input volume control ;
P17V_MIXER_AC97_SRP_REC_VOL_L	equ 0x65    ; SRP Record to Mixer_AC97 input volume control ;
P17V_MIXER_AC97_SRP_REC_VOL_H	equ 0x66    ; SRP Record to Mixer_AC97 input volume control ;
; 67 - 68: Reserved ;
P17V_MIXER_Spdif_10K1_VOL_L	equ 0x69    ; 10K to Mixer_Spdif input volume control ;
P17V_MIXER_Spdif_10K1_VOL_H	equ 0x6A    ; 10K to Mixer_Spdif input volume control ;
P17V_MIXER_Spdif_P17V_VOL_L	equ 0x6B    ; P17V to Mixer_Spdif input volume control ;
P17V_MIXER_Spdif_P17V_VOL_H	equ 0x6C    ; P17V to Mixer_Spdif input volume control ;
P17V_MIXER_Spdif_SRP_REC_VOL_L	equ 0x6D    ; SRP Record to Mixer_Spdif input volume control ;
P17V_MIXER_Spdif_SRP_REC_VOL_H	equ 0x6E    ; SRP Record to Mixer_Spdif input volume control ;
; 6f - 70: Reserved ;
P17V_MIXER_I2S_10K1_VOL_L	equ 0x71    ; 10K to Mixer_I2S input volume control ;
P17V_MIXER_I2S_10K1_VOL_H	equ 0x72    ; 10K to Mixer_I2S input volume control ;
P17V_MIXER_I2S_P17V_VOL_L	equ 0x73    ; P17V to Mixer_I2S input volume control ;
P17V_MIXER_I2S_P17V_VOL_H	equ 0x74    ; P17V to Mixer_I2S input volume control ;
P17V_MIXER_I2S_SRP_REC_VOL_L	equ 0x75    ; SRP Record to Mixer_I2S input volume control ;
P17V_MIXER_I2S_SRP_REC_VOL_H	equ 0x76    ; SRP Record to Mixer_I2S input volume control ;
; 77 - 78: Reserved ;
P17V_MIXER_AC97_ENABLE		equ 0x79    ; Mixer AC97 input audio enable ;
P17V_MIXER_SPDIF_ENABLE 	equ 0x7A    ; Mixer SPDIF input audio enable ;
P17V_MIXER_I2S_ENABLE		equ 0x7B    ; Mixer I2S input audio enable ;
P17V_AUDIO_OUT_ENABLE		equ 0x7C    ; Audio out enable ;
P17V_MIXER_ATT			equ 0x7D    ; SRP Mixer Attenuation Select ;
P17V_SRP_RECORD_SRR		equ 0x7E    ; SRP Record channel source Select ;
P17V_SOFT_RESET_SRP_MIXER	equ 0x7F    ; SRP and mixer soft reset ;

P17V_AC97_OUT_MASTER_VOL_L	equ 0x80    ; AC97 Output master volume control ;
P17V_AC97_OUT_MASTER_VOL_H	equ 0x81    ; AC97 Output master volume control ;
P17V_SPDIF_OUT_MASTER_VOL_L	equ 0x82    ; SPDIF Output master volume control ;
P17V_SPDIF_OUT_MASTER_VOL_H	equ 0x83    ; SPDIF Output master volume control ;
P17V_I2S_OUT_MASTER_VOL_L	equ 0x84    ; I2S Output master volume control ;
P17V_I2S_OUT_MASTER_VOL_H	equ 0x85    ; I2S Output master volume control ;
; 86 - 87: Not used ;
P17V_I2S_CHANNEL_SWAP_PHASE_INVERSE	equ 0x88    ; I2S out mono channel swap
							 ; and phase inverse ;
P17V_SPDIF_CHANNEL_SWAP_PHASE_INVERSE	equ 0x89    ; SPDIF out mono channel swap
							 ; and phase inverse ;
; 8A: Not used ;
P17V_SRP_P17V_ESR		equ 0x8B    ; SRP_P17V estimated sample rate and rate lock ;
P17V_SRP_REC_ESR		equ 0x8C    ; SRP_REC estimated sample rate and rate lock ;
P17V_SRP_BYPASS 		equ 0x8D    ; srps channel bypass and srps bypass ;
; 8E - 92: Not used ;
P17V_I2S_SRC_SEL		equ 0x93    ; I2SIN mode sel ;






